# -*- coding: utf-8 -*-
# =============================================================================
## @file BenderBox.py
#  Bender application for Ganga  (CMAKE-based version)
#  Actually there are three applications under the single hat
#  - BenderModule : classic Bender application: use a full power of bender
#  - BenderRun    : execute scripts in the context of ``bender'' environment
#  - OstapRun     : execute scripts in the context of ``ostap'' environment
#  @author Vladimir ROMANOVSKY Vladimir.Romanovskiy@cern.ch
#  @author Vanya BELYAEV  Ivan.Belyaev@itep.ru
# =============================================================================
"""Bender application for Ganga  (CMAKE-based version)
Actually there are three applications under the single hat
- BenderModule : classic Bender application: use a full power of bender
- BenderScript : execute scripts in the context of ``bender'' environment
- Ostap : execute scripts in the context of ``ostap'' environment
"""
# =============================================================================
import os
import tempfile 
from   GangaCore.GPIDev.Lib.File.File                       import ShareDir  
from   GangaCore.GPIDev.Schema                              import Schema , Version , SimpleItem , GangaFileItem
from   GangaCore.Utility.logging                            import getLogger
from   GangaCore.GPIDev.Adapters.ApplicationRuntimeHandlers import allHandlers


from   GangaLHCb.Lib.Applications.GaudiExecUtils        import prepare_cmake_app
from   GangaLHCb.Lib.RTHandlers.GaudiExecRTHandlers     import GaudiExecDiracRTHandler, GaudiExecRTHandler
from   GangaLHCb.Lib.Applications.GaudiExec             import GaudiExec

logger = getLogger()
# =============================================================================
## wrapper python script for Bender module
_script_ = """#!/usr/bin/env python
# -*- coding: utf-8 -*-
# =============================================================================
from Gaudi.Configuration import importOptions
importOptions('{datafile}')
from Configurables import LHCbApp
LHCbApp().XMLSummary='summary.xml'
from Gaudi.Configuration import EventSelector,FileCatalog
from copy import deepcopy
inputdata    = deepcopy( EventSelector() .Input    )
filecatalogs = deepcopy( FileCatalog  () .Catalogs )
EventSelector().Input    = [] ## reset it
FileCatalog  ().Catalogs = [] ## reset it
# =============================================================================
# configure Bender and run it
# =============================================================================
import  sys
sys.path = sys.path + ['.']
import {modulename} as USERMODULE
USERMODULE.configure( inputdata , filecatalogs {paramstring})
USERMODULE.run({events})
# =============================================================================
# The END
# =============================================================================
"""
# =============================================================================
## wrapper python script for Bender script
_script_bender_ = """#!/usr/bin/env python
# -*- coding: utf-8 -*-
# =============================================================================
from distutils.spawn import find_executable
bender_script = find_executable('bender')
import sys
from Configurables import LHCbApp
LHCbApp().XMLSummary='summary.xml'
# =============================================================================
## redefine arguments
# =============================================================================
sys.argv   += {scripts}
sys.argv   += [ '--no-color' ] ## automatically added argument
sys.argv   += {arguments}
sys.argv   += [ '--import'   ] + {imports}
sys.argv   += [ '--no-castor'] ## automatically added argument
sys.argv   += [ '--import={datafile}' ]
sys.argv   += [ '--batch'    ] ##  automatically added argument
sys.argv   += [ '--command'  ] + {command}
##
# =============================================================================
## execute bender script
# =============================================================================
import runpy
runpy.run_path ( bender_script, init_globals = globals() , run_name = '__main__' )
# =============================================================================
# The END
# =============================================================================
"""
# =============================================================================
## wrapper python script for Ostap script
_script_ostap_ = """#!/usr/bin/env python
# -*- coding: utf-8 -*-
# =============================================================================
from distutils.spawn import find_executable
ostap_script = find_executable('ostap')
import sys
from Configurables import LHCbApp
LHCbApp().XMLSummary='summary.xml'
# =============================================================================
## redefine arguments
# =============================================================================
sys.argv += {scripts}
sys.argv += [ '--no-color' ] ## automatically added argument
sys.argv += {arguments}
sys.argv += [ '--batch'    ] ## automatically added argument
sys.argv += [ '--command'  ] + {command}
##
# =============================================================================
## execute bender script
# =============================================================================
import runpy
runpy.run_path ( ostap_script, init_globals = globals() , run_name = '__main__' )
# =============================================================================
# The END
# =============================================================================
"""
# =============================================================================

# =============================================================================
## @class BenderModule
#  The main application to run ``classic'' Bender module with the proper
#  ``configure'' and ``run'' methods
#
#  User needs to supply:
#  - the name of Bender module to run
#  - dictionary of parameters to be forwarder to <code>configure</code> method
#  - number of event to process
#
#  Usage:
#  @code
#  j.application = BenderModule ( module    = 'the_path/the_module.py' ,
#                                 events    = 1000  ,
#                                 params    = {...} ,
#                                 directory =  ...  )
#  @endcode
#  @author Vladimir ROMANOVSKY Vladimir.Romanovskiy@cern.ch
#  @author Vanya BELYAEV       Ivan.Belyaev@itep.ru
class BenderModule(GaudiExec):
    """ The main application to run ``classic'' Bender (module with the proper
    ``configure'' and ``run'' methods)

    User needs to supply:
    - the name of Bender module to run
    - dictionary of parameters to be forwarder to <code>configure</code> method
    - number of event to process

    ======
    Usage:
    ======

    j.application = BenderModule ( module    = 'the_path/the_module.py' ,
                                   events    = 1000  ,
                                   params    = {...} ,
                                   directory =  ...  )

    """
    ##
    _schema = GaudiExec._schema.inherit_copy()
    _schema.version.major += 0
    _schema.version.minor += 0
    ## make entries
    for key,val in _schema.datadict.iteritems() :
        if key == 'useGaudiRun'  : val._update( { 'defvalue' : False } )
        if key == 'getMetadata'  : val._update( { 'defvalue' : True } )
        if not key in ( 'platform' , 'directory', 'getMetadata') :
            if not val['hidden'] : val._update( { 'hidden'   : 1 } )
    ## add new entries
    _schema.datadict [ 'module' ] = GangaFileItem (
        optional  = 0 ,
        doc       = """The file with Bender module. It is expected that module contains the methods ``configure'' & ``run'' with the proper signatures""")
    _schema.datadict [ 'params' ] = SimpleItem    (
        defvalue  = {} , typelist = ['dict', 'str', 'int', 'bool', 'float'] ,
        doc       = """The dictionary of parameters to be forwarded to ``configure'' method of the supplied Bender module""")
    _schema.datadict [ 'events' ] = SimpleItem    ( defvalue=-1, typelist=['int'], doc= "Number of events to process" )
    ##
    _category      = 'applications'
    _name          = 'BenderModule'
    _exportmethods = ['prepare', 'unprepare' ]

    # =========================================================================
    def configure(self, masterjobconfig):
        self.options = [ self.module ]
        return (None,None)

    # =========================================================================
    def getWNPythonContents(self):
        """Return the wrapper script which is used to run Bender on the WN
        """
        f = self.module
        file_name       = os.path.basename ( os.path.join  ( f.localDir , f.namePattern  ) )
        module_name     = file_name.split('.')[0]
        param_string    = ',params=%s' % self.params if self.params else ''
        data_file       = GaudiExecDiracRTHandler.data_file

        return _script_.format (
            datafile    = data_file ,
            modulename  = module_name ,
            paramstring = param_string ,
            events      = self.events
            )

# =============================================================================
## @class BenderRun
#  The main application to run ``BenderScript''
#
#  User needs to supply:
#  - the scripts to be executed
#  - the configuration scripts (aka ``options'') to be imported  (optional)
#  - bender interactive commands to be executed                  (optional)
#  - other arguments for script ``bender''                       (optional)
#
#  Usage:
#  @code
#  j.application = BenderRun    ( scripts   = [ 'the_path/the_module.py' ] ,
#                                 directory =  ...  )
#  @endcode
#
#  The actual command to be executed is:
#  @code
#  > bender [ scripts [ scripts ...  --no-color [ arguments ] --import [ imports [ imports [ ... --no-castor --import=data.py --batch   [ --command  [ commands [ ...
#  @endcode
#
#  @author Vladimir ROMANOVSKY Vladimir.Romanovskiy@cern.ch
#  @author Vanya BELYAEV       Ivan.Belyaev@itep.ru
class BenderRun(GaudiExec):
    """The main application to run ``BenderScript''

    User needs to supply:
    - the scripts to be executed
    - the configuration scripts (aka ``options'') to be imported  (optional)
    - bender interactive commands to be executed                  (optional)
    - other arguments for script ``bender''                       (optional)

    The actual command to be executed is:

    > bender [ scripts [ scripts ...  --no-color [ arguments ] --import [ imports [ imports [ ... --no-castor --import=data.py --batch   [ --command  [ commands [ ...

    ======
    Usage:
    ======

    j.application = BenderRun    ( scripts   = [ 'the_path/the_module.py'     ] ,
                                   imports   = [ 'some_miport_file.py'        ] ,
                                   commands  = [ 'ls()' , 'run(10)'  , 'ls()' ] ,
                                   arguments = [ ... ] ,
                                   directory =  ...  )

    """
    _schema = GaudiExec._schema.inherit_copy()
    _schema.version.major += 0
    _schema.version.minor += 0
    ## make entries
    for key,val in _schema.datadict.iteritems() :
        if key == 'useGaudiRun'  : val._update( { 'defvalue' : False } )
        if key == 'getMetadata'  : val._update( { 'defvalue' : True } )
        if not key in ( 'platform' , 'directory', 'getMetadata') :
            if not val['hidden'] : val._update( { 'hidden'   : 1 } )
    ## add new entries
    _schema.datadict [ 'scripts'   ] = GangaFileItem   (
        optional        = 0       ,
        sequence        = 1       ,
        strict_sequence = 0       ,
        doc             = """The names of the script files to execute. A copy will be made at submission time. The script are executed within ``bender'' context""")
    _schema.datadict [ 'imports'   ] = GangaFileItem   (
        defvalue        = []      ,
        sequence        = 1       ,
        strict_sequence = 0       ,
        doc             = """The names of the configurtaion scripts (ana ``options'') to be imported via ``importOptions''. A copy will be made at submission time""")
    _schema.datadict [ 'commands'  ] = SimpleItem (
        defvalue        = []      ,
        typelist        = ['str'] ,
        sequence        = 1       ,
        strict_sequence = 0       ,
        doc             = """The bender commands to be executed, e.g. [ 'run(10)' , 'print ls()' , 'print dir()' ]""")
    _schema.datadict [ 'arguments' ] = SimpleItem (
        defvalue        = []      ,
        typelist        = ['str'] ,
        sequence        = 1       ,
        strict_sequence = 0       ,
        doc             = """The list of command-line arguments for bender script, e.g. ['-w','-p5'], etc. Following arguments will be appended automatically:  --no-color, --no-castor and --batch.""")
    ##
    _category      = 'applications'
    _name          = 'BenderRun'
    _exportmethods = ['prepare', 'unprepare' ]

    # =========================================================================
    def configure(self, masterjobconfig):
        self.options = [ f for f in self.scripts ] + [ f for f in self.imports ]
        return (None,None)

    # =========================================================================
    def getWNPythonContents(self):
        """Return the wrapper script which is used to run BenderScript on the WN
        """
        data_file       = GaudiExecDiracRTHandler.data_file
        return _script_bender_.format (
            scripts   = [ os.path.basename ( os.path.join ( f.localDir , f.namePattern  ) ) for f in self.scripts ] ,
            arguments = self.arguments ,
            imports   = [ os.path.basename ( os.path.join ( f.localDir , f.namePattern  ) ) for f in self.imports ] ,
            datafile  = data_file ,
            command   = self.commands
            )

# =============================================================================
## @class OstapRun
#  The main application to run ``Ostap''
#
#  User needs to supply:
#  - the scripts to be executed
#  - bender interactive commands to be executed                  (optional)
#  - other arguments for script ``ostap''                        (optional)
#
#  The actual command to be executed is:
#  @code
#  > ostap [ scripts [scripts [scripts ...  [ arguments ] --no-color --batch   [ --command  [ commands [ commands [ commands
#  @endcode
#  Usage:
#  @code
#  j.application = OstapRun ( scripts   = ['path_to_script/the_script.py']  ,
#                             arguments = [ '--no-canvas' ]  ,
#                             commands  = [ 'getMetadataprint dir()' ]  ) )
#  @encode
#  @author Vladimir ROMANOVSKY Vladimir.Romanovskiy@cern.ch
#  @author Vanya BELYAEV  Ivan.Belyaev@itep.ru
class OstapRun(GaudiExec):
    """The main application to run ``Ostap''

    User needs to supply:
    - the scripts to be executed
    - ostap interactive commands to be executed                  (optional)
    - other arguments for script ``ostap''                       (optional)

    The actual command to be executed is:
    > ostap [ scripts [scripts [scripts ...  [ arguments ] --no-color --batch   [ --command  [ commands [ commands [ commands

    ======
    Usage:
    ======

    j.application = OstapRun ( scripts   = ['path_to_script/the_script.py']  ,
                               arguments = [ '--no-canvas' ]  ,
                               commands  = [ 'print dir()' ]  ) )
    """
    _schema = GaudiExec._schema.inherit_copy()
    _schema.version.major += 0
    _schema.version.minor += 0
    ## make entries
    for key,val in _schema.datadict.iteritems() :
        if key == 'useGaudiRun'  : val._update( { 'defvalue' : False } )
        if key == 'getMetadata'  : val._update( { 'defvalue' : True } )
        if not key in ( 'platform' , 'directory', 'getMetadata') :
            if not val['hidden'] : val._update( { 'hidden'   : 1 } )
    ## add new entries
    _schema.datadict [ 'scripts'   ] = GangaFileItem   (
        optional        = 0       ,
        sequence        = 1       ,
        strict_sequence = 0       ,
        doc             = """The names of ostap script files to be executed. The files are executed within ``ostap'' context. A copy will be made at submission time""")
    _schema.datadict [ 'commands'  ] = SimpleItem (
        defvalue        = []      ,
        typelist        = ['str'] ,
        sequence        = 1       ,
        strict_sequence = 0       ,
        doc             = """The ostap commands to be executed, e.g. [ 'print dir()' ]""")
    _schema.datadict [ 'arguments' ] = SimpleItem (
        defvalue        = []      ,
        typelist        = ['str'] ,
        sequence        = 1       ,
        strict_sequence = 0       ,
        doc             = "The list of command-line arguments for ``ostap'' script, e.g. ['-w','-p5'], etc. Following arguments are appended automatically:  --no-color and --batch""")
    ##
    _category      = 'applications'
    _name          = 'OstapRun'
    _exportmethods = ['prepare', 'unprepare' ]

    # =========================================================================
    def configure(self, masterjobconfig):
        self.options = [ f for f in self.scripts ]
        return (None,None)

    # =========================================================================
    def getWNPythonContents(self):
        """Return the wrapper script which is used to run Ostap on the WN
        """
        data_file       = GaudiExecDiracRTHandler.data_file
        return _script_ostap_.format (
            scripts   = [ os.path.basename ( os.path.join ( f.localDir , f.namePattern  ) ) for f in self.scripts ] ,
            arguments = self.arguments ,
            command   = self.commands
            )


# =============================================================================
## prepare Bender application
#  - specify the path:
#  @code
#  j = Job ( application = prepareBender ( 'v30r1' , module = 'the_path/the_module.py' , path = '$HOME/cmtuser' ) )
#  @endcode
#  - use the temporary directory
#  @code
#  j = Job ( application = prepareBender ( 'v30r1' , module = 'the_path/the_modue.py' , use_tmp = True  ) )
#  @endcode
#  - other parameters  can be added as keyword arguments
#  @code
#  j = Job ( application = prepareBender ( 'v30r1' , module = 'the_path/the_modue.py' , use_tmp = True , params = {} , events = 100 ) )
#  @endcode
def prepareBender ( version , module , path = '$HOME/cmtuser', use_tmp = False , **kwargs ) :
    """Prepare Bender application
    - specify the path:
    >>> j = Job ( application = prepareBender ( 'v30r1' , module = 'the_path/the_module.py' , path = '$HOME/cmtuser' ) )
    - use the temporary directory
    >>> j = Job ( application = prepareBender ( 'v30r1' , module = 'the_path/the_modue.py' , use_tmp = True  ) )
    - other parameters  can be added as keyword arguments
    >>> j = Job ( application = prepareBender ( 'v30r1' , module = 'the_path/the_modue.py' , use_tmp = True , params = {} , events = 100 ) )
    """
    if use_tmp or not path :
        path = tempfile.mkdtemp ( prefix = 'GANGA_' )
        logger.info ( 'Bender application will be prepared in the temporary directory %s' % path )

    the_path = prepare_cmake_app ( 'Bender' , version , path )

    from GangaCore.GPI import BenderModule as _BM  ## black magic
    return _BM ( module = module , directory = the_path , **kwargs )

# =============================================================================
## prepare BenderRun application
#  - specify the script
#  @code
#  j = Job ( application = prepareBenderRun ( 'v30r1' , script = 'the_path/the_script.py') )
#  @endcode
#  - specify the scripts
#  @code
#  j = Job ( application = prepareBenderRun ( 'v30r1' , scripts = ['the_path/the_script.py' , 'another_script.py' ] ) )
#  @endcode
#  - specify the directory explicitely:
#  @code
#  j = Job ( application = prepareBenderRun ( 'v30r1' , path = '$HOME/mydir' , script = 'the_script.py' )
#  @endcode
#  - use the temporary directory
#  @code
#  j = Job ( application = prepareBenderRun ( 'v30r1' , use_tmp = True , script = 'the_script.py' )
#  @endcode
#  - one can also use the configuration parameters:
#  @code
#  j = Job ( application = prepareBender ( 'v30r1' , script = 'the_path/the_script.py' , commands = [] , imports = []  ) )
#  @endcode
def prepareBenderRun ( version , script = '' , path = '$HOME/cmtuser', use_tmp = False , **kwargs ) :
    """Prepare BenderRun application
    - specify the script
    >>> j = Job ( application = prepareBenderRun ( 'v30r1' , script = 'the_path/the_script.py') )
    - specify the scripts
    >>> j = Job ( application = prepareBenderRun ( 'v30r1' , scripts = ['the_path/the_script.py' , 'another_script.py' ] ) )
    - specify the directory explicitely:
    >>> j = Job ( application = prepareBenderRun ( 'v30r1' , path = '$HOME/mydir' , script = 'the_script.py' )
    - use the temporary directory
    >>> j = Job ( application = prepareBenderRun ( 'v30r1' , use_tmp = True , script = 'the_script.py' )
    One can also use the configuration parameters:
    >>> j = Job ( application = prepareBender ( 'v30r1' , script = 'the_path/the_script.py' , commands = [] , imports = []  ) )
    """
    if use_tmp or not path :
        path = tempfile.mkdtemp ( prefix = 'GANGA_' )
        logger.info ( 'Bender application will be prepared in the temporary directory %s' % path )

    the_path = prepare_cmake_app ( 'Bender' , version , path )
    #
    scripts = kwargs.pop('scripts', [] )
    if script : scripts.append ( script )
    #

    from GangaCore.GPI import BenderRun as _BR  ## black magic    
    return _BR ( scripts = scripts , directory = the_path , **kwargs )


# =============================================================================
## prepare OstapRun application
#  - specify the script
#  @code
#  j = Job ( application = prepareOstapRun ( 'v30r1' , script = 'the_path/the_script.py') )
#  @endcode
#  - specify the scripts
#  @code
#  j = Job ( application = prepareOstapRun ( 'v30r1' , scripts = ['the_path/the_script.py' , 'another_script.py' ] ) )
#  @endcode
#  - specify the directory explicitely:
#  @code
#  j = Job ( application = prepareOstapRun ( 'v30r1' , path = '$HOME/mydir' , script = 'the_script.py' )
#  @endcode
#  - use the temporary directory
#  @code
#  j = Job ( application = prepareOstapRun ( 'v30r1' , use_tmp = True , script = 'the_script.py' )
#  @endcode
#  - one can also use the configuration parameters:
#  @code
#  j = Job ( application = prepareOstapRun ( 'v30r1' , script = 'the_path/the_script.py' , commands = [] , imports = []  ) )
#  @endcode
#  - use another project  ('Analysis' is minimal one... , 'Bender' is default one)
#  @code
#  j = Job ( application = prepareOstapRun ( project = 'Analysis' , version = 'v18r0' , script = 'the_path/the_script.py' , commands = [] ) )
#  @endcode
def prepareOstapRun ( version , script = '' , path = '$HOME/cmtuser', use_tmp = False , project = 'Bender' , **kwargs ) :
    """Prepare OstapRun application
    - specify the script
    >>> j = Job ( application = prepareOstapRun ( 'v30r1' , script = 'the_path/the_script.py') )
    - specify the scripts 
    >>> j = Job ( application = prepareOstapRun ( 'v30r1' , scripts = ['the_path/the_script.py' , 'another_script.py' ] ) )
    - specify the directory explicitely:
    >>> j = Job ( application = prepareOstapRun ( 'v30r1' , path = '$HOME/mydir' , script = 'the_script.py' )
    - use the temporary directory
    >>> j = Job ( application = prepareOstapRun ( 'v30r1' , use_tmp = True , script = 'the_script.py' )
    - one can also use the configuration parameters:
    >>> j = Job ( application = prepareOstapRun ( 'v30r1' , script = 'the_path/the_script.py' , commands = [] , imports = []  ) )
    - use another project  ('Analysis' is minimal one...)
    >>> j = Job ( application = prepareOstapRun ( project = 'Analysis' , version = 'v18r0' , script = 'the_path/the_script.py' , commands = []  ) )
    """
    if use_tmp or not path :
        path = tempfile.mkdtemp ( prefix = 'GANGA_' )
        logger.info ( 'Bender application will be prepared in the temporary directory %s' % path )

    the_path = prepare_cmake_app ( project , version , path )
    #
    scripts = kwargs.pop('scripts', [] )
    if script : scripts.append ( script )
    #
    from GangaCore.GPI import OstapRun as _OR  ## black magic    
    return _OR ( scripts = scripts , directory = the_path , **kwargs )

## export it! 
from GangaCore.Runtime.GPIexport import exportToGPI
exportToGPI ( 'prepareBender'      , prepareBender       , 'Functions' )
exportToGPI ( 'prepareBenderRun'   , prepareBenderRun    , 'Functions' )
exportToGPI ( 'prepareOstapRun'    , prepareOstapRun     , 'Functions' )

##
for app in ( 'BenderModule' , 'BenderRun' , 'OstapRun' ) :
    allHandlers.add ( app , 'Dirac', GaudiExecDiracRTHandler)
    for backend in ("Local","Interactive","LSF","PBS","SGE","Condor"):
        allHandlers.add ( app , backend, GaudiExecRTHandler)

# =============================================================================
# The END
# =============================================================================



from GangaCore.GPIDev.Lib.Dataset import Dataset
from GangaCore.GPIDev.Schema import Schema, SimpleItem, Version
from GangaCMS.Lib.ConfParams import CMSSW, CRAB, GRID, USER


class CRABDataset(Dataset):
    """Dataset definition for CRAB jobsets."""
    schemadic = {}
    schemadic.update(CMSSW().schemadic)
    schemadic.update(CRAB().schemadic)
    schemadic.update(GRID().schemadic)
    schemadic.update(USER().schemadic)
    schemadic['target_site'] = SimpleItem(defvalue=None,
                                          typelist=['type(None)', 'str'],
                                          doc='Target site name for the job.')
    _schema = Schema(Version(1, 0), schemadic)
    _category = 'datasets'
    _name = 'CRABDataset'

    def __init__(self):
        super(CRABDataset, self).__init__()



#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#
'''Application handler for Gaudi applications in LHCb.'''
import os
import tempfile
import gzip
import pickle
from GangaCore.GPIDev.Schema import SimpleItem, FileItem
from GangaCore.Core.exceptions import ApplicationConfigurationError
import GangaCore.Utility.logging
from GangaCore.Core.GangaRepository import getRegistry
from GangaCore.GPIDev.Lib.File import ShareDir
from GangaCore.GPIDev.Lib.Registry.PrepRegistry import ShareRef
from GangaCore.Utility.util import unique
from GangaCore.GPIDev.Base.Proxy import GPIProxyObjectFactory, isType
from GangaCore.GPIDev.Adapters.StandardJobConfig import StandardJobConfig
from GangaCore.Utility.Config import getConfig
from GangaCore.Utility.files import expandfilename
from GaudiBase import GaudiBase
from GangaCore.Utility.files import fullpath
from GangaGaudi.Lib.Applications.GaudiUtils import gzipFile
logger = GangaCore.Utility.logging.getLogger()

#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#


def GaudiDocString(appname):
    "Provide the documentation string for each of the Gaudi based applications"

    doc = """The Gaudi Application handler

    The Gaudi application handler is for running LHCb GAUDI framework
    jobs. For its configuration it needs to know the version of the application
    and what options file to use. More detailed configuration options are
    described in the schema below.

    An example of submitting a Gaudi job to Dirac could be:

    app = Gaudi(version='v99r0')

    # Give absolute path to options file. If several files are given, they are
    # just appended to each other.
    app.optsfile = ['/afs/...../myopts.opts']

    # Append two extra lines to the python options file
    app.extraopts=\"\"\"
    ApplicationMgr.HistogramPersistency ="ROOT"
    ApplicationMgr.EvtMax = 100
    \"\"\"

    # Define dataset
    ds = LHCbDataset(['LFN:foo','LFN:bar'])

    # Construct and submit job object
    j=Job(application=app,backend=Dirac())
    j.submit()

    """
    return doc.replace("Gaudi", appname)


#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#

class Gaudi(GaudiBase):

    _name = 'Gaudi'
    __doc__ = GaudiDocString(_name)
    _category = 'applications'
    _exportmethods = GaudiBase._exportmethods[:]
    _exportmethods += ['prepare', 'unprepare']
    _hidden = 1
    _schema = GaudiBase._schema.inherit_copy()

    docstr = 'The gaudirun.py cli args that will be passed at run-time'
    _schema.datadict['args'] = SimpleItem(defvalue=['-T'], sequence=1, strict_sequence=0,
                                          typelist=['str', 'type(None)'], doc=docstr)
    docstr = 'The name of the optionsfile. Import statements in the file ' \
             'will be expanded at submission time and a full copy made'
    _schema.datadict['optsfile'] = FileItem(preparable=1, sequence=1, strict_sequence=0, defvalue=[],
                                            doc=docstr)
    docstr = 'A python configurable string that will be appended to the '  \
             'end of the options file. Can be multiline by using a '  \
             'notation like \nHistogramPersistencySvc().OutputFile = '  \
             '\"myPlots.root"\\nEventSelector().PrintFreq = 100\n or by '  \
             'using triple quotes around a multiline string.'
    _schema.datadict['extraopts'] = SimpleItem(preparable=1, defvalue=None,
                                               typelist=['str', 'type(None)'], doc=docstr)

    _schema.version.major += 0
    _schema.version.minor += 0

    def _auto__init__(self):
        """bootstrap Gaudi applications. If called via a subclass
        set up some basic structure like version platform..."""
        self._init()

    def _parse_options(self):
        raise NotImplementedError

    def prepare(self, force=False):

        from GangaCore.GPIDev.Lib.GangaList.GangaList import GangaList
        from GangaCore.GPIDev.Lib.File.File import File
        if isType(self.optsfile, (list, tuple, GangaList)):
            for this_file in self.optsfile:
                if type(this_file) is str:
                    this_file = File(this_file)
                else:
                    continue

        elif type(self.optsfile) is str:
            self.optsfile = [File(self.optsfile)]

        try:
            super(Gaudi, self).prepare(force)
        except Exception, err:
            logger.debug("Super Prepare Error:\n%s" % str(err))
            raise err

        logger.debug("Prepare")

        _is_prepared = self.is_prepared

        #logger.info("_is_prepared: %s" % _is_prepared)

        share_dir = os.path.join(expandfilename(getConfig('Configuration')['gangadir']),
                                 'shared',
                                 getConfig('Configuration')['user'],
                                 _is_prepared.name)

        # We will return a list of files 'send_to_share' which will be copied into the jobs
        # inputsandbox when prepare called from job object. NOTE that these files will not go
        # into an inputsandbox when prepare called on standalone app.
        # Things in the inputsandbox end up in the working dir at runtime.

        # Exception is just re-thrown here after setting is_prepared to None
        # could have done the setting in the actual functions but didnt want
        # prepared state altered from the readInputData pseudo-static member
        try:
            self._check_inputs()
        except Exception, err:
            logger.debug("_check_inputs Error:\n%s" % str(err))
            self.unprepare()
            raise err

        # write env into input dir and share dir
        share_path = os.path.join(share_dir, 'debug')
        if not os.path.isdir(share_path):
            os.makedirs(share_path)
        this_file = gzip.GzipFile(os.path.join(share_path, 'gaudi-env.py.gz'), 'wb')
        this_file.write('gaudi_env = %s' % str(self.getenv(True)))
        this_file.close()

        self._parse_options()
        #try:
        #    self._parse_options()
        #except Exception, err:
        #    logger.debug("_parse_options Error:\n%s" % str(err))
        #    self.unprepare()
        #    raise err

        gzipFile(os.path.join(share_dir, 'inputsandbox', '_input_sandbox_%s.tar' % self.is_prepared.name),
                 os.path.join(share_dir, 'inputsandbox', '_input_sandbox_%s.tgz' % self.is_prepared.name),
                 True)
        # add the newly created shared directory into the metadata system if
        # the app is associated with a persisted object
        self.checkPreparedHasParent(self)
        self.post_prepare()  # create hash

    def _check_inputs(self):
        """Checks the validity of some of user's entries for Gaudi schema"""

        # Warn user that no optsfiles given
        if len(self.optsfile) == 0:
            logger.warning("The 'optsfile' is not set. I hope this is OK!")
        else:
            # Check for non-exting optsfiles defined
            nonexistentOptFiles = []
            for f in self.optsfile:
                from GangaCore.GPIDev.Lib.File.File import File
                if type(f) is str:
                    myFile = File(f)
                else:
                    myFile = f
                myFile.name = fullpath(myFile.name)
                if not os.path.isfile(myFile.name):
                    nonexistentOptFiles.append(myFile)

            if len(nonexistentOptFiles):
                tmpmsg = "The 'optsfile' attribute contains non-existent file/s: ["
                for _f in nonexistentOptFiles:
                    tmpmsg += "'%s', " % _f.name
                msg = tmpmsg[:-2] + ']'
                raise ApplicationConfigurationError(msg)

    def master_configure(self):
        '''Handles all common master_configure actions.'''
        return (None, StandardJobConfig())

    def configure(self, appmasterconfig):
        return (None, StandardJobConfig())


#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#




def license():
    'Print the full license (GPL)'
    from GangaCore.Utility.logging import getLogger
    logger = getLogger()
    from os import path
    from GangaCore import _gangaPythonPath
    with open(path.join(_gangaPythonPath, '..', 'LICENSE_GPL')) as printable:
        logger.info(printable.read())

def typename(obj):
    """Return a name of Ganga object as a string, example: typename(j.application) -> 'DaVinci'"""
    from GangaCore.GPIDev.Base.Proxy import isProxy, stripProxy, implRef
    if isProxy(obj):
        if hasattr(stripProxy(obj), '_name'):
            return stripProxy(obj)._name
        else:
            from GangaCore.Utility.logging import getLogger
            logger = getLogger()
            logger.error("Object %s DOES NOT have the _name parameter set" % (str(obj)))
            return ""
    else:
        if hasattr(obj, '_name'):
            return obj._name
        else:
            from GangaCore.Utility.logging import getLogger
            logger = getLogger()
            logger.error("Object %s DOES NOT have the %s or _name parameter set" % (str(obj), str(implRef)))
            return ""

def categoryname(obj):
    """Return a category of Ganga object as a string, example: categoryname(j.application) -> 'applications'"""
    from GangaCore.GPIDev.Base.Proxy import isProxy, stripProxy, implRef
    if isProxy(obj):
        if hasattr(stripProxy(obj), '_category'):
            return stripProxy(obj)._category
        else:
            from GangaCore.Utility.logging import getLogger
            logger = getLogger()
            logger.error("Object %s DOES NOT have the _category parameter set" % (str(obj)))
            return ""
    else:
        if hasattr(obj, '_category'):
            return obj._category
        else:
            from GangaCore.Utility.logging import getLogger
            logger = getLogger()
            logger.error("Object %s DOES NOT have the %s or _category parameter set" % (str(obj), str(implRef)))
            return ""

def plugins(category=None):
    """List loaded plugins.

    If no argument is given return a dictionary of all loaded plugins.
    Keys are category name. Values are lists of plugin names in each
    category.

    If a category is specified (for example 'splitters') return a list
    of all plugin names in this category.
    """
    from GangaCore.Utility.Plugin import allPlugins
    if category:
        return allPlugins.allClasses(category).keys()
    else:
        d = {}
        for c in allPlugins.allCategories():
            d[c] = allPlugins.allCategories()[c].keys()
        return d

# FIXME: DEPRECATED
def list_plugins(category):
    """List all plugins in a given category, OBSOLETE: use plugins(category)"""
    raise DeprecationWarning("use plugins('%s')" % category)

def applications():
    """return a list of all available applications, OBSOLETE: use plugins('applications')"""
    raise DeprecationWarning("use plugins('applications')")

def backends():
    """return a list of all available backends, OBSOLETE: use plugins('backends')"""
    raise DeprecationWarning("use plugins('backends')")

# FIXME: END

def convert_merger_to_postprocessor(j):
    from GangaCore.GPIDev.Base.Proxy import stripProxy
    if len(stripProxy(j.postprocessors).process_objects):
        logger.info('job(%s) already has postprocessors' % j.fqid)
    if stripProxy(j).merger is None:
        logger.info('job(%s) does not have a merger to convert' % j.fqid)
    if not len(stripProxy(j.postprocessors).process_objects) and stripProxy(j).merger is not None:
        mp = MultiPostProcessor()
        mp.process_objects.append(stripProxy(j).merger)
        stripProxy(j).postprocessors = mp




from __future__ import absolute_import

import os
import subprocess
import tempfile
from datetime import datetime, timedelta

import GangaCore.Utility.logging
from GangaCore.GPIDev.Schema import SimpleItem
from GangaCore.Utility import GridShell
from GangaCore.Utility.Config import getConfig

from GangaCore.GPIDev.Adapters.ICredentialInfo import ICredentialInfo, cache, retry_command
from GangaCore.GPIDev.Adapters.ICredentialRequirement import ICredentialRequirement
from GangaCore.Core.exceptions import CredentialRenewalError, InvalidCredentialError

logger = GangaCore.Utility.logging.getLogger()


class VomsProxyInfo(ICredentialInfo):
    """
    A wrapper around a voms proxy file
    """

    def __init__(self, requirements, check_file=False, create=False):
        """
        Args:
            requirements (ICredentialRequirement): An object specifying the requirements
            check_file (bool): Raise an exception if the file does not exist
            create (bool): Create the credential file
        """

        self._shell = None

        super(VomsProxyInfo, self).__init__(requirements, check_file, create)

    @retry_command
    def create(self):
        """
        Creates the grid proxy.

        Raises:
            CredentialRenewalError: If the renewal process returns a non-zero value
        """
        voms_command = ''
        logger.debug('require ' + str(self.initial_requirements.vo))
        if self.initial_requirements.vo:
            voms_command = '-voms %s' % self.initial_requirements.vo
            if self.initial_requirements.group or self.initial_requirements.role:
                voms_command += ':'
                if self.initial_requirements.group:
                    voms_command += '/%s' % self.initial_requirements.group
                if self.initial_requirements.role:
                    voms_command += '/%s' % self.initial_requirements.role
        logger.debug(voms_command)
        command = 'voms-proxy-init -out "%s" %s' % (self.location, voms_command)
        logger.debug(command)
        if not self.shell:
            raise CredentialRenewalError('Failed to create VOMS proxy due to configuration problem')
        try:
            self.shell.check_call(command)
        except subprocess.CalledProcessError:
            raise CredentialRenewalError('Failed to create VOMS proxy')
        else:
            logger.debug('Grid proxy %s created. Valid for %s', self.location, self.time_left())

    @property
    def shell(self):
        """
        This creates a grid shell instance which is used by the VomsProxy only when needed
        """
        if self._shell is None:
            self._shell = GridShell.getShell()
        return self._shell

    def destroy(self):
        """
        This destroys the voms proxy on disk
        """
        self.shell.cmd1('voms-proxy-destroy -file "%s"' % self.location, allowed_exit=[0, 1])

        if os.path.isfile(self.location):
            os.remove(self.location)

    @cache
    def info(self):
        """
        This returns the full proxy info with all information from a given proxy on disk
        """
        status, output, message = self.shell.cmd1('voms-proxy-info -all -file "%s"' % self.location)
        return output

    @property
    @cache
    def identity(self):
        """
        This returns the identity associated with the voms proxy on disk
        """
        status, output, message = self.shell.cmd1('voms-proxy-info -file "%s" -identity' % self.location)
        return output.strip()

    @property
    @cache
    def vo(self):
        """
        This returns the vo associated with a voms proxy on disk
        """
        status, output, message = self.shell.cmd1('voms-proxy-info -file "%s" -vo' % self.location)
        if status != 0:
            return None
        return output.split(':')[0].strip()

    @property
    @cache
    def role(self):
        """
        This returns the role associated with a voms proxy on disk
        """
        status, output, message = self.shell.cmd1('voms-proxy-info -file "%s" -vo' % self.location)
        if status != 0:
            return None  # No VO
        vo_list = output.split(':')
        if len(vo_list) <= 1:
            return None  # No command after VO
        return vo_list[1].split('/')[-1].split('=')[-1].strip()

    @property
    @cache
    def group(self):
        """
        This returns the group associated with a voms proxy on disk
        """
        status, output, message = self.shell.cmd1('voms-proxy-info -file "%s" -vo' % self.location)
        if status != 0:
            return None  # No VO
        vo_list = output.split(':')
        if len(vo_list) <= 1:
            return None  # No command after VO
        # TODO Make this support multiple groups and subgroups
        group_role_list = vo_list[1].split('/')
        if len(group_role_list) <= 2:
            return None  # No group specified in command
        return group_role_list[-1].strip()

    @cache
    def expiry_time(self):
        """
        This returns the time that a proxy will expire at in seconds
        """
        status, output, message = self.shell.cmd1('voms-proxy-info -file "%s" -timeleft' % self.location)
        if status != 0:
            return datetime.now()
        return datetime.now() + timedelta(seconds=int(output))

    def default_location(self):
        """
        This returns the default_location of a voms proxy on disk based only on the location and uid
        """
        return os.getenv('X509_USER_PROXY') or os.path.join(tempfile.gettempdir(), 'x509up_u'+str(os.getuid()))


class VomsProxy(ICredentialRequirement):
    """
    An object specifying the requirements of a VOMS proxy file
    """
    _schema = ICredentialRequirement._schema.inherit_copy()
    _schema.datadict['identity'] = SimpleItem(defvalue=None, typelist=[str, None], doc='Identity for the proxy')
    _schema.datadict['vo'] = SimpleItem(defvalue=None, typelist=[str, None], doc='Virtual Organisation for the proxy. Defaults to LGC/VirtualOrganisation')
    _schema.datadict['role'] = SimpleItem(defvalue=None, typelist=[str, None], doc='Role that the proxy must have')
    _schema.datadict['group'] = SimpleItem(defvalue=None, typelist=[str, None], doc='Group for the proxy - either "group" or "group/subgroup"')

    _category = 'CredentialRequirement'

    info_class = VomsProxyInfo

    def __init__(self, **kwargs):
        """
        Construct a voms proxy requirement and assign the default vo from the config if none has been provided
        """
        super(VomsProxy, self).__init__(**kwargs)
        if 'vo' not in kwargs and getConfig('LCG')['VirtualOrganisation']:
            self.vo = getConfig('LCG')['VirtualOrganisation']

    def encoded(self):
        """
        This returns the additional encoding of the identity, vo, role and group which are to be encoded into the voms file location
        """
        return ':'.join(requirement for requirement in [self.identity, self.vo, self.role, self.group] if requirement)  # filter out the empties




"""
Definition of GPI base objects and proxies.
"""

from GangaCore.GPIDev.Base.Objects import GangaObject
from GangaCore.Core.exceptions import GangaException
from GangaCore.GPIDev.Base.Proxy import GangaAttributeError, ProtectedAttributeError, ReadOnlyObjectError



# Definition of config - as it comes in the plugin code

import GangaCore.Utility.Config
cc = GangaCore.Utility.Config.makeConfig('TestConfig2','more testing stuff')

cc.addOption('str1','','doc')
cc.addOption('str2','','doc')
cc.addOption('str3','','doc')
cc.addOption('str4','','doc')
cc.addOption('str5','','doc')
cc.addOption('str6','','doc')
cc.addOption('str7','','doc')
cc.addOption('str8','','doc')

cc.addOption('int1',0,'doc')
cc.addOption('int2',0,'doc')
cc.addOption('int3',0,'doc')
cc.addOption('int4',0,'doc')
cc.addOption('int5',0,'doc')
cc.addOption('int6',0,'doc')
cc.addOption('int7',0,'doc')
cc.addOption('int8',0,'doc')

cc.addOption('none1',None,'doc')
cc.addOption('none2',None,'doc')
cc.addOption('none3',None,'doc')
cc.addOption('none4',None,'doc')
cc.addOption('none5',None,'doc')
cc.addOption('none6',None,'doc')
cc.addOption('none7',None,'doc')
cc.addOption('none8',None,'doc')

cc.addOption('list1',[],'doc')
cc.addOption('list2',[],'doc')
cc.addOption('list3',[],'doc')
cc.addOption('list4',[],'doc')
cc.addOption('list5',[],'doc')
cc.addOption('list6',[],'doc')
cc.addOption('list7',[],'doc')
cc.addOption('list8',[],'doc')



###############################################################################
# Ganga Project. http://cern.ch/ganga
#
# $Id: PACKAGE.py,v 1.1 2008-10-04 17:42:37 karl Exp $
###############################################################################
""" Refer to Ganga/PACKAGE.py for details on the purpose of this module.
"""

external_packages = {}

from GangaCore.Utility.Setup import PackageSetup

setup = PackageSetup( external_packages )

def standardSetup( setup=setup ):
    pass



"""
Sandbox functions used in the job wrapper script on the worker node.
The text of this module is sourced into the job wrapper script.
It therefore may use ###TAGS###  which are expanded in the wrapper script.
"""

INPUT_TARBALL_NAME = '_input_sandbox.tgz'
OUTPUT_TARBALL_NAME = '_output_sandbox.tgz'
PYTHON_DIR = '_python'

import os
import mimetypes

import tarfile
from contextlib import closing

def getPackedInputSandbox(tarpath, dest_dir='.'):
    """Get all sandbox_files from tarball and write them to the workdir.
       This function is called by wrapper script at the run time.
    Arguments:
      'tarpath': a path to the tarball
      'dest_dir': a destination directory
    """

    try:
        with closing(tarfile.open(tarpath, "r:*")) as tf:
            tf.extractall(dest_dir)
    except:
        raise Exception("Error opening tar file: %s" % tarpath)


def createOutputSandbox(output_patterns, filter, dest_dir):
    """Get all files matching output patterns except filtered with filter and
       write them to the destination directory.
       This function is called by wrapper script at the run time.
    Arguments:
      'output_patterns': list of filenames or patterns.
      'filter': function to filter files (return True to except) 
      'dest_dir': destination directory for output files
    """

    try:
        from files import multi_glob, recursive_copy
    except (IOError, ImportError) as e:
        import sys

        print("Failed to import files")
        print("sys:")
        print(sys.path)
        print("env:")
        print(os.environ)
        print("ls:")
        print(os.listdir("."))
        print("pattern:")
        print(output_patterns)
        print("destdir:")
        print(dest_dir)

        try:
            import traceback
            traceback.print_stack()
        except:
            pass

        print("Trying fix")
        sys.path.insert(0, os.path.join(os.getcwd(), PYTHON_DIR))

        try:
            from files import multi_glob, recursive_copy
            print("Success!")
        except (IOError, ImportError) as e:
            print("Fail!")
            raise e

    for f in multi_glob(output_patterns, filter):
        try:
            if not os.path.exists(dest_dir):
                os.makedirs(dest_dir)
            recursive_copy(f, dest_dir)
        except Exception as x:
            print("ERROR: (job ###JOBID### createOutput )", x)


def createPackedOutputSandbox(output_patterns, _filter, dest_dir):
    """Get all files matching output patterns except filtered with filter and
       put them to the Sandbox tarball in destination directory.
       This function is called by wrapper script at the run time.
    Arguments:
      'output_patterns': list of filenames or patterns.
      'filter': function to filter files (return True to except) 
      'dest_dir': destination directory for tarball
    """

    tgzfile = os.path.join(dest_dir, OUTPUT_TARBALL_NAME)

    try:
        from files import multi_glob, recursive_copy
    except (IOError, ImportError) as e:
        import sys

        print("Failed to import files")
        print("sys:")
        print(sys.path)
        print("env:")
        print(os.environ)
        print("ls:")
        print(os.listdir("."))
        print("pattern:")
        print(output_patterns)
        print("destdir:")
        print(dest_dir)

        try:
            import traceback
            traceback.print_stack()
        except:
            pass

        print("Trying fix")
        sys.path.insert(0, os.path.join(os.getcwd(), PYTHON_DIR))

        try:
            from files import multi_glob, recursive_copy
            print("Success!")
        except (IOError, ImportError) as e:
            print("Fail!")
            raise e

    outputlist = multi_glob(output_patterns, _filter)

    if outputlist:
        if mimetypes.guess_type(tgzfile)[1] in ['gzip']:
            file_format = 'gz'
        elif mimetypes.guess_type(tgzfile)[1] in ['bzip2']:
            file_format = 'bz2'
        else:
            file_format = ''
        with closing(tarfile.open(tgzfile, "w:%s" % file_format)) as tf:
            tf.dereference = True
            for f in outputlist:
                tf.add(f)




import os
import os.path
import re
import datetime
from os.path import exists, isdir, realpath, isfile, islink
from os import pathsep, listdir, environ, fdopen
import subprocess
import tempfile
import GangaCore.Utility.logging
import GangaCore.Utility.Config
from optparse import OptionParser, OptionValueError

from GangaCore.Utility.Config.Config import _after_bootstrap
from GangaCore.Utility.logging import getLogger

from GangaCore.Runtime.GPIexport import exportToGPI

from GangaCore.GPIDev.Credentials.CredentialStore import credential_store
from GangaDirac.Lib.Credentials.DiracProxy import DiracProxy
from GangaLHCb.Utility.LHCbDIRACenv import store_dirac_environment

logger = getLogger()

if not _after_bootstrap:
    configLHCb = GangaCore.Utility.Config.makeConfig('LHCb', 'Parameters for LHCb')

    # Set default values for the LHCb config section.
    dscrpt = 'The name of the local site to be used for resolving LFNs into PFNs.'
    configLHCb.addOption('LocalSite', '', dscrpt)

    dscrpt = 'Files from these services will go to the output sandbox (unless \
    overridden by the user in a specific job via the Job.outputdata field). Files \
    from all other known handlers will go to output data (unless overridden by \
    the user in a specific job via the Job.outputsandbox field).'
    configLHCb.addOption('outputsandbox_types',
                         ['CounterSummarySvc', 'NTupleSvc',
                          'HistogramPersistencySvc', 'MicroDSTStream',
                          'EvtTupleSvc'], dscrpt)
    dscrpt = 'The string that is added after the filename in the options to tell' \
             ' Gaudi how to read the data. This is the default value used if the '\
             'file name does not match any of the patterns in '\
             'datatype_string_patterns.'
    configLHCb.addOption('datatype_string_default',
                         """TYP='POOL_ROOTTREE' OPT='READ'""", dscrpt)
    dscrpt = 'If a file matches one of these patterns, then the string here '\
             'overrides the datatype_string_default value.'
    defval = {"SVC='LHCb::MDFSelector'": ['*.raw', '*.RAW', '*.mdf', '*.MDF']}
    configLHCb.addOption('datatype_string_patterns', defval, dscrpt)
    configLHCb.addOption('UserAddedApplications', "", "List of user added LHCb applications split by ':'")

    configLHCb.addOption('SplitByFilesBackend', 'OfflineGangaDiracSplitter',
                         'Possible SplitByFiles backend algorithms to use to split jobs into subjobs,\
                          options are: GangaDiracSplitter, OfflineGangaDiracSplitter, \
                          splitInputDataBySize and splitInputData')
    defaultLHCbDirac = 'prod'
    configLHCb.addOption('LHCbDiracVersion', defaultLHCbDirac, 'set LHCbDirac version')


def _store_root_version():
    if 'ROOTSYS' in os.environ:
        vstart = os.environ['ROOTSYS'].find('ROOT/') + 5
        vend = os.environ['ROOTSYS'][vstart:].find('/')
        rootversion = os.environ['ROOTSYS'][vstart:vstart + vend]
        os.environ['ROOTVERSION'] = rootversion
    else:
        msg = 'Tried to setup ROOTVERSION environment variable but no ROOTSYS variable found.'
        raise OptionValueError(msg)


if not _after_bootstrap:
    store_dirac_environment()
    _store_root_version()


def standardSetup():

    import PACKAGE
    PACKAGE.standardSetup()


def loadPlugins(config=None):
    logger.debug("Importing Backends")
    import Lib.Backends
    logger.debug("Importing Applications")
    import Lib.Applications
    logger.debug("Importing LHCbDataset")
    import Lib.LHCbDataset
    logger.debug("Importing Mergers")
    import Lib.Mergers
    logger.debug("Importing RTHandlers")
    import Lib.RTHandlers
    logger.debug("Importing Splitters")
    import Lib.Splitters
    logger.debug("Importing Tasks")
    import Lib.Tasks
    logger.debug("Importing Files")
    import Lib.Files
    logger.debug("Importing Checkers")
    import Lib.Checkers
    logger.debug("Importing LHCbTasks")
    import Lib.Tasks
    logger.debug("Finished Importing")


def postBootstrapHook():
    configDirac = GangaCore.Utility.Config.getConfig('DIRAC')
    configOutput = GangaCore.Utility.Config.getConfig('Output')
    configPoll = GangaCore.Utility.Config.getConfig('PollThread')
    configProxy = GangaCore.Utility.Config.getConfig('defaults_DiracProxy')

    configDirac.setSessionValue('DiracEnvJSON', os.environ['GANGADIRACENVIRONMENT'])
    configDirac.setSessionValue('userVO', 'lhcb')
    configDirac.setSessionValue('allDiracSE', ['CERN-USER', 'CNAF-USER', 'GRIDKA-USER', 'RRCKI-USER',
                                               'IN2P3-USER', 'SARA-USER', 'PIC-USER', 'RAL-USER'])
    configDirac.setSessionValue('noInputDataBannedSites', [])
    configDirac.setSessionValue('RequireDefaultSE', False)
    configDirac.setSessionValue('proxyInitCmd', 'lhcb-proxy-init')
    configDirac.setSessionValue('proxyInfoCmd', 'lhcb-proxy-info')

    configOutput.setSessionValue('FailJobIfNoOutputMatched', 'False')

    configPoll.setSessionValue('autoCheckCredentials', False)

    configProxy.setSessionValue('group', 'lhcb_user')
    configProxy.setSessionValue('encodeDefaultProxyFileName', False)

# This is being dropped from 6.1.0 due to causing some bug in loading large numbers of jobs
#
# This will be nice to re-add once there is lazy loading support passed to the display for the 'jobs' command 09/2015 rcurrie
#
#from GangaCore.GPIDev.Lib.Registry.JobRegistry import config as display_config
#display_config.setSessionValue( 'jobs_columns', ('fqid', 'status', 'name', 'subjobs', 'application', 'backend', 'backend.actualCE', 'backend.extraInfo', 'comment') )
#display_config.setSessionValue( 'jobs_columns_functions', {'comment': 'lambda j: j.comment', 'backend.extraInfo': 'lambda j : j.backend.extraInfo ', 'subjobs': 'lambda j: len(j.subjobs)', 'backend.actualCE': 'lambda j:j.backend.actualCE', 'application': 'lambda j: j.application._name', 'backend': 'lambda j:j.backend._name'} )
#display_config.setSessionValue('jobs_columns_width', {'fqid': 8, 'status': 10, 'name': 10, 'application': 15, 'backend.extraInfo': 30, 'subjobs': 8, 'backend.actualCE': 17, 'comment': 20, 'backend': 15} )

    from GangaCore.Core.GangaThread.WorkerThreads import getQueues
    queue = getQueues()
    if queue is not None:
        queue.add(updateCreds)
    else:
        updateCreds()

def updateCreds():
    try:
        for group in ('lhcb_user', ):
            if group == 'lhcb_user':
                credential_store[DiracProxy(group=group, encodeDefaultProxyFileName=False)]
            credential_store[DiracProxy(group=group)]
    except KeyError:
        pass

class gridProxy(object):
    """
    This is a stub class which wraps functions from the `credential_store` sentinal to familiar functions from Ganga 6.2 and prior
    """

    @classmethod
    def renew(cls):
        """
        This method is similar to calling::

            credential_store.create(DiracProxy())

        or::

            credential_store[DiracProxy()].renew()

        as appropriate.
        """

        from GangaCore.GPI import credential_store, DiracProxy
        try:
            cred = credential_store[DiracProxy()]
            if not cred.is_valid():
                cred.create()
        except KeyError:
            credential_store.create(DiracProxy())

    @classmethod
    def create(cls):
        """
        This is a wrapper for::

            credential_store.create(DiracProxy())
        """
        cls.renew()

    @classmethod
    def destroy(cls):
        """
        This is a wrapper for::

            credential_store[DiracProxy()].destroy()
        """
        from GangaCore.GPI import credential_store, DiracProxy
        try:
            cred = credential_store[DiracProxy()]
            cred.destroy()
        except KeyError:
            pass

exportToGPI('gridProxy', gridProxy, 'Functions')



##########################################################################
# Ganga Project. http://cern.ch/ganga
#
# $Id$
##########################################################################

from GangaCore.Core.exceptions import ApplicationConfigurationError
from GangaCore.GPIDev.Base.Proxy import isType
from GangaCore.GPIDev.Schema import Schema, Version, SimpleItem
from GangaCore.GPIDev.Adapters.ISplitter import ISplitter
from GangaCore.GPIDev.Lib.Dataset import GangaDataset



class GangaDatasetSplitter(ISplitter):

    """ Split job based on files given in GangaDataset inputdata field """
    _name = "GangaDatasetSplitter"
    _schema = Schema(Version(1, 0), {
        'files_per_subjob': SimpleItem(defvalue=5, doc='the number of files per subjob', typelist=[int]),

        'maxFiles': SimpleItem(defvalue=-1,
                               doc='Maximum number of files to use in a masterjob (None or -1 = all files)',
                               typelist=[int, None]),
    })

    def split(self, job):
        subjobs = []

        if not job.inputdata or not isType(job.inputdata, GangaDataset):
            raise ApplicationConfigurationError(
                "No GangaDataset given for GangaDatasetSplitter")

        # find the full file list
        full_list = []
        for f in job.inputdata.files:

            if f.containsWildcards():
                # we have a wildcard so grab the subfiles
                for sf in f.getSubFiles(process_wildcards=True):
                    full_list.append(sf)
            else:
                # no wildcards so just add the file
                full_list.append(f)

        if len(full_list) == 0:
            raise ApplicationConfigurationError(
                "GangaDatasetSplitter couldn't find any files to split over")

        masterType = type(job.inputdata)

        # split based on all the sub files
        fid = 0
        subjobs = []
        filesToRun = len(full_list)
        if not self.maxFiles == -1:
            filesToRun = min(self.maxFiles, filesToRun)
        while fid < filesToRun:
            j = self.createSubjob(job)
            j.inputdata = masterType()
            j.inputdata.treat_as_inputfiles = job.inputdata.treat_as_inputfiles
            for sf in full_list[fid:fid + self.files_per_subjob]:
                j.inputdata.files.append(sf)

            fid += self.files_per_subjob
            subjobs.append(j)

        return subjobs



###############################################################################
# Ganga Project. http://cern.ch/ganga
#
# $Id: PDF_LCG.py,v 1.1 2009-05-10 16:42:03 karl Exp $
###############################################################################
# File: PDF_LCG.py
# Author: K. Harrison
# Created: 070126

"""Module containing class dealing with preparation of jobs to run
   PDF application on LCG backend"""

__author__  = "K.Harrison <Harrison@hep.phy.cam.ac.uk>"
__date__    = "30 November 2007"
__version__ = "1.1"

from PDF_Local import PDF_Local
from PDF_Local import ptime
from GangaCore.GPIDev.Lib.File import  File
from GangaCore.Lib.LCG import LCGJobConfig
from GangaCore.Lib.LCG import LCGRequirements
from GangaCore.Utility import logging
from GangaCore.Utility.files import fullpath

import os

logger = logging.getLogger()

class PDF_LCG( PDF_Local ):

   def prepare( self, app, appsubconfig, appmasterconfig, jobmasterconfig ):

      local = PDF_Local.prepare\
         ( self, app, appsubconfig, appmasterconfig, jobmasterconfig )
      inbox = []
      inbox.extend( local.inputbox )
      requirements = LCGRequirements()
      tag = "VO-camont-Camtology-%s" % appsubconfig[ "version" ]
      requirements.software = [ tag ]
      lcg = LCGJobConfig( exe = local.exe, inputbox = inbox,\
         outputbox = local.outputbox, requirements = requirements )
      return lcg

   def tail( self, job ):
      lineList, outbox = PDF_Local.tail( self )
     # Remove last lines (completion times) for local jobs
      lineList = lineList[ : -4 ]
     # Remove tarball from outbox, as this is to be uploaded to storage element
      outbox.remove( "media.tar.gz" ) 

      gridUrl = os.path.join( job.outputdata.getGridStorage(), "media.tar.gz" )
      lineList.extend\
         ( [
         "",
         "UPLOAD_START_TIME=\\",
         "%s" % ptime,
         "edg-gridftp-mkdir --parents %s" % os.path.dirname( gridUrl ),
         "globus-url-copy file:${WORKDIR}/media.tar.gz %s" % gridUrl,
         "UPLOAD_END_TIME=\\",
         "%s" % ptime,
         "",
         "echo \"Upload_start: ${UPLOAD_START_TIME}\" >> ${RUN_DATA}",
         "echo \"Upload_end: ${UPLOAD_END_TIME}\" >> ${RUN_DATA}",
         "JOB_END_TIME=\\",
         "%s" % ptime,
         "echo \"Job_end: ${JOB_END_TIME}\" >> ${RUN_DATA}",
         "echo \"End time: $(date)\"",
         ] )
      return ( lineList, outbox )



"""Base finisher IAction implementation.

The BaseFinisher class provides an abstract finisher implementation.
 
"""

from GangaRobot.Framework.Action import IAction
from GangaRobot.Framework import Utility
from GangaCore.Utility.logging import getLogger
from GangaCore.GPI import *
import time

logger = getLogger()


class BaseFinisher(IAction):
    
    """Base finisher IAction implementation.
    
    An abstract action implementation providing a basis for concrete finisher
    implementations. It contains a chain of finishers which are processed in the
    execute() method by calling handleisfinished() for each job associated with
    the runid, on each finisher in turn.

    Implementing a sub-class involves doing any of the following:
    - Override the __init__() method to add additional finishers to the chain
    attribute.
    - Override the handleisfinished() method to indicate that the given job is
    finished.
    
    """

    def __init__(self):
        """Create a new base finisher.
        
        The chain attribute, a list of finishers, is initialised to [self].
        
        Sub-classes can override this constructor to set the chain attribute.
        e.g.
        self.chain = [CoreFinisher(), self]

        """
        self.chain = [self]

    def execute(self, runid):
        """Invoke each of the finishers in the chain.
        
        Keyword arguments:
        runid -- A UTC ID string which identifies the run.
        
        Loops until the run is finished or the elapsed time exceeds the
        configurable BaseFinisher_Timeout (seconds).
        
        Each job in the jobtree directory named by the runid, e.g.
        /2007-06-25_09.18.46, is passed to the handleisfinished() method of each
        finisher in the chain. If any finisher returns False, for any job,
        then the run is considered unfinished.
        
        The sleep period in the loop is 1/10th the timeout, constrained to the
        range [10, 60] seconds.
        
        """
        logger.info("Finishing jobs for run '%s'.", runid)
        startsecs = time.time()
        timeoutsecs = self.getoption('BaseFinisher_Timeout')
        sleepsecs = timeoutsecs / 10
        if sleepsecs < 10: sleepsecs = 10
        if sleepsecs > 60: sleepsecs = 60
        
        logger.info('Waiting for jobs to finish. Timeout %d seconds.'
                    ' Sleep period %d seconds', timeoutsecs, sleepsecs)

        path = Utility.jobtreepath(runid)
        jobs = jobtree.getjobs(path)
        
        while True:
            #suppose finished is True
            finished = True
            for j in jobs:
                #if one job not finished then finished is False
                for finisher in self.chain:
                    if not finisher.handleisfinished(j):
                        finished = False
                        break
            elapsedsecs = time.time() - startsecs
            # test break condition here to avoid sleep after satisfied condition
            if finished or timeoutsecs < elapsedsecs:
                break
            # break condition on satisfied, so sleep
            time.sleep(sleepsecs)
        
        if finished:
            logger.info('All jobs finished after %d seconds.', elapsedsecs)
        else:
            logger.info('Timeout after %d seconds.', elapsedsecs)
            
    def handleisfinished(self, job):
        """Default implementation returns False.
        
        Keyword arguments:
        job -- A Ganga Job associated with the run.
        
        Sub-classes can override this method to indicate if the given job is
        finished. Return True if the job is finished, False otherwise.
        e.g.
        def handleisfinished(self, job):
            return job.status in ['completed', 'killed', 'failed']

        This method is called in the context of a Ganga session.
        
        """
        return False
    



import os
from os.path import join, expanduser
from optparse import OptionValueError
from distutils.version import LooseVersion

from GangaDirac.Lib.Utilities.DiracUtilities import write_env_cache
import GangaCore.Utility.Config
from GangaCore.Utility.execute import execute
from GangaCore.Utility.logging import getLogger
from GangaCore.Core.exceptions import PluginError

logger = getLogger()


def store_dirac_environment():
    """Store the LHCbDIRAC environment in a cache file."""


# Re-enable test case in TestLHCbDiracVersion.py when re-enabled.

#    platform_env_var = 'CMTCONFIG'
#    try:
#        platform = os.environ[platform_env_var]
#    except KeyError:
#        logger.error("Environment variable %s is missing. Can't cache LHCbDIRAC environment.", platform_env_var)
#        raise PluginError
    # While LHCbDirac is only available for gcc49 we shall unfortunately hard-code the platform.
    platform = 'x86_64-slc6-gcc49-opt'

    requestedVersion = GangaCore.Utility.Config.getConfig('LHCb')['LHCbDiracVersion']
    # by default the version is 'prod', and in this case we need to resolve the actual version
    # if a specific version is requested, then we can simply try to determine its environment

    # this returns a list, like ['prod'] or ['v', 9, 'r', 3, 'p', 19]
    lbVersion = LooseVersion(requestedVersion).version
    try:
        # we check if a real version (e.g. v9r2, or v9r3p13 or v10r12-pre9) is requested
        if isinstance(lbVersion[1], int) and isinstance(lbVersion[3], int):
            logger.warn("Specific version is requested (%s), please consider removing it!", requestedVersion)
            diracversion = requestedVersion
        else:
            diracversion = 'prod'
    except IndexError:  # here we assume 'prod', and honestly nothing else should be here.
        diracversion = 'prod'

    fdir = join(expanduser("~/.cache/Ganga/GangaLHCb"), platform)
    fname = join(fdir, diracversion)

    # FIXME: for v9r3+ lb-run works in compatibility mode
    # (sourcing the environment is what should be done for v9r3+)
    cmd = 'lb-run -c best LHCBDIRAC/{version} python -c "import os; print(dict(os.environ))"'.format(version=diracversion)
    env = execute(cmd)

    if isinstance(env, str):
        try:
            env_temp = eval(env)
            env = env_temp

        except SyntaxError:
            logger.error("LHCbDirac version %s does not exist", diracversion)
            raise OptionValueError("LHCbDirac version {version} does not exist".format(version=diracversion))
    try:
        write_env_cache(env, fname)
        logger.debug("Storing new LHCbDirac environment (%s:%s)", str(diracversion), str(platform))
    except (OSError, IOError, TypeError):
        logger.error("Unable to store LHCbDirac environment")
        raise PluginError
    logger.info("Using LHCbDirac version %s", diracversion)
    os.environ['GANGADIRACENVIRONMENT'] = fname



"""Base submitter IAction implementation.

The BaseSubmitter class provides an abstract submitter implementation.
 
"""

from GangaRobot.Framework.Action import IAction
from GangaRobot.Framework import Utility
from GangaCore.Utility.logging import getLogger
from GangaCore.GPI import *

logger = getLogger()


class BaseSubmitter(IAction):
    
    """Base submitter IAction implementation.
    
    An abstract action implementation providing a basis for concrete job
    submitter implementations. It contains a chain of submitters which are
    processed in the execute() method by calling handlesubmit() on each
    submitter in turn.
    
    Implementing a sub-class involves doing any of the following:
    - Override the __init__() method to add additional submitters to the chain
    attribute.
    - Override the handlesubmit() method to submit jobs using the Ganga GPI.
    
    """

    def __init__(self):
        """Create a new base submitter.
        
        The chain attribute, a list of submitters, is initialised to [self].
        
        Sub-classes can override this constructor to set the chain attribute.
        e.g.
        self.chain = [CoreSubmitter(), self]

        """
        self.chain = [self]

    def execute(self, runid):
        """Invoke each of the submitters in the chain.
        
        Keyword arguments:
        runid -- A UTC ID string which identifies the run.
        
        An empty list of jobids is created and passed to the handlesubmit()
        method of each the submitter in the chain.
        
        Any job, whose id is added to the argument jobids, is added to the
        jobtree in the directory named by the runid, e.g. /2007-06-25_09.18.46.

        """
        logger.info("Submitting jobs for run '%s'.", runid)
        jobids = []
        for submitter in self.chain:
            submitter.handlesubmit(jobids, runid)
        path = Utility.jobtreepath(runid)
        jobtree.mkdir(path)
        for id in jobids:
            jobtree.add(jobs(id), path)
        
        logger.info("%d submitted jobs added to jobtree path '%s'.", len(jobids), path)
        
        
    def handlesubmit(self, jobids, runid):
        """Empty default implementation.
        
        Keyword arguments:
        jobids -- A list of the submitted job ids.
        runid -- A UTC ID string which identifies the run.
        
        Sub-classes can override this method to submit jobs using the Ganga GPI.
        The id of any submitted job should be added to the jobids list.
        e.g.
        def handlesubmit(self, jobids, runid):
            j = Job()
            j.application.is_prepared = True
            j.submit()
            jobids.append(j.id)

        This method is called in the context of a Ganga session.
        
        """
        pass



#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#
'''Application handler for GaudiPython applications in LHCb.'''
import os
import pprint
from os.path import split, join
import inspect
from GangaCore.GPIDev.Schema import FileItem, SimpleItem
import GangaCore.Utility.logging
from GangaCore.GPIDev.Lib.File import File
from GangaCore.Utility.util import unique
from GangaCore.GPIDev.Lib.File import ShareDir
from GangaCore.GPIDev.Lib.File.FileBuffer import FileBuffer
from GangaGaudi.Lib.Applications.GaudiBase import GaudiBase
from GangaGaudi.Lib.Applications.GaudiUtils import fillPackedSandbox, gzipFile
from GangaCore.Core.exceptions import ApplicationConfigurationError
from GangaCore.Utility.files import expandfilename, fullpath
from GangaCore.Utility.Config import getConfig
from GangaCore.Utility.Shell import Shell
from AppsBaseUtils import guess_version
from GangaCore.GPIDev.Adapters.StandardJobConfig import StandardJobConfig
import shutil
import tempfile

# Added for XML PostProcessing
from GangaLHCb.Lib.RTHandlers.RTHUtils import getXMLSummaryScript
from GangaLHCb.Lib.Applications import XMLPostProcessor

logger = GangaCore.Utility.logging.getLogger()

#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#


class GaudiPython(GaudiBase):

    """The GaudiPython Application handler

    The GaudiPython application handler is for running LHCb GaudiPython
    jobs. This means running scripts where you are in control of the events
    loop etc. If you are usually running jobs using the gaudirun script
    this is *not* the application handler you should use. Instead use the
    DaVinci, Gauss, ... handlers.

    For its configuration it needs to know what application and version to
    use for setting up the environment. More detailed configuration options
    are described in the schema below.

    An example of submitting a GaudiPython job to Dirac could be:

    app = GaudiPython(project='DaVinci', version='v19r14')

    # Give absolute path to the python file to be executed. 
    # If several files are given the subsequent ones will go into the
    # sandbox but it is the users responsibility to include them
    app.script = ['/afs/...../myscript.py']

    # Define dataset
    ds = LHCbDataset(['LFN:spam','LFN:eggs'])

    # Construct and submit job object
    j=Job(application=app,backend=Dirac(),inputdata=ds)
    j.submit()

"""
    _name = 'GaudiPython'
    _category = 'applications'
    _exportmethods = GaudiBase._exportmethods[:]
    _exportmethods += ['prepare', 'unprepare']

    _schema = GaudiBase._schema.inherit_copy()
    docstr = 'The package the application belongs to (e.g. "Sim", "Phys")'
    _schema.datadict['package'] = SimpleItem(defvalue=None,
                                             typelist=['str', 'type(None)'],
                                             doc=docstr)
    docstr = 'The package where your top level requirements file is read '  \
             'from. Can be written either as a path '  \
             '\"Tutorial/Analysis/v6r0\" or in traditional notation '  \
             '\"Analysis v6r0 Tutorial\"'
    _schema.datadict['masterpackage'] = SimpleItem(defvalue=None,
                                                   typelist=[
                                                       'str', 'type(None)'],
                                                   doc=docstr)
    docstr = 'Extra options to be passed onto the SetupProject command '\
             'used for configuring the environment. As an example '\
             'setting it to \'--dev\' will give access to the DEV area. '\
             'For full documentation of the available options see '\
             'https://twiki.cern.ch/twiki/bin/view/LHCb/SetupProject'
    _schema.datadict['setupProjectOptions'] = SimpleItem(defvalue='',
                                                         typelist=[
                                                             'str', 'type(None)'],
                                                         doc=docstr)
    docstr = 'The name of the script to execute. A copy will be made ' + \
             'at submission time'
    _schema.datadict['script'] = FileItem(preparable=1, sequence=1, strict_sequence=0, defvalue=[],
                                          doc=docstr)
    docstr = "List of arguments for the script"
    _schema.datadict['args'] = SimpleItem(defvalue=[], typelist=['str'],
                                          sequence=1, doc=docstr)
    docstr = 'The name of the Gaudi application (e.g. "DaVinci", "Gauss"...)'
    _schema.datadict['project'] = SimpleItem(preparable=1, defvalue=None,
                                             typelist=['str', 'type(None)'],
                                             doc=docstr)
    _schema.version.major += 2
    _schema.version.minor += 0

    def _get_default_version(self, gaudi_app):
        return guess_version(self, gaudi_app)

    def _attribute_filter__set__(self, n, v):
        if n == 'project':
            self.appname = v
        return v

    def _auto__init__(self):
        if (not self.appname) and (not self.project):
            self.project = 'DaVinci'  # default
        if (not self.appname):
            self.appname = self.project
        self._init()

    def _getshell(self):
        import EnvironFunctions
        return EnvironFunctions._getshell(self)

    def prepare(self, force=False):
        super(GaudiPython, self).prepare(force)
        self._check_inputs()

        share_dir = os.path.join(expandfilename(getConfig('Configuration')['gangadir']), 'shared', getConfig('Configuration')['user'], self.is_prepared.name)

        fillPackedSandbox(self.script, os.path.join(share_dir, 'inputsandbox', '_input_sandbox_%s.tar' % self.is_prepared.name))
        gzipFile(os.path.join(share_dir, 'inputsandbox', '_input_sandbox_%s.tar' % self.is_prepared.name),
                 os.path.join(share_dir, 'inputsandbox', '_input_sandbox_%s.tgz' % self.is_prepared.name), True)
        # add the newly created shared directory into the metadata system if
        # the app is associated with a persisted object
        self.checkPreparedHasParent(self)
        self.post_prepare()

    def master_configure(self):
        return (None, StandardJobConfig())

    def configure(self, master_appconfig):
        # self._configure()
        name = join('.', self.script[0].subdir, split(self.script[0].name)[-1])
        script = "from Gaudi.Configuration import *\n"
        if self.args:
            script += 'import sys\nsys.argv += %s\n' % str(self.args)
        script += "importOptions('data.py')\n"
        script += "execfile(\'%s\')\n" % name

        # add summary.xml
        outputsandbox_temp = XMLPostProcessor._XMLJobFiles()
        outputsandbox_temp += unique(self.getJobObject().outputsandbox)
        outputsandbox = unique(outputsandbox_temp)

        input_files = []
        input_files += [FileBuffer('gaudipython-wrapper.py', script)]
        logger.debug("Returning Job Configuration")
        return (None, StandardJobConfig(inputbox=input_files,
                                        outputbox=outputsandbox))

    def _check_inputs(self):
        """Checks the validity of user's entries for GaudiPython schema"""
        if len(self.script) == 0:
            logger.warning("No script defined. Will use a default "
                           'script which is probably not what you want.')
            self.script = [File(os.path.join(
                os.path.dirname(inspect.getsourcefile(GaudiPython)),
                'options/GaudiPythonExample.py'))]
        else:
            for f in self.script:
                f.name = fullpath(f.name)

        return

    def postprocess(self):
        XMLPostProcessor.postprocess(self, logger)

#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#

# Associate the correct run-time handlers to GaudiPython for various backends.

from GangaCore.GPIDev.Adapters.ApplicationRuntimeHandlers import allHandlers
from GangaLHCb.Lib.RTHandlers.LHCbGaudiRunTimeHandler import LHCbGaudiRunTimeHandler
from GangaLHCb.Lib.RTHandlers.LHCbGaudiDiracRunTimeHandler import LHCbGaudiDiracRunTimeHandler

for backend in ['LSF', 'Interactive', 'PBS', 'SGE', 'Local', 'Condor', 'Remote']:
    allHandlers.add('GaudiPython', backend, LHCbGaudiRunTimeHandler)
allHandlers.add('GaudiPython', 'Dirac', LHCbGaudiDiracRunTimeHandler)

#\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\#



"""CheckForPreRelease  IAction implementation"""

from GangaRobot.Framework.Action import IAction
from GangaRobot.Framework import Utility
from GangaRobot.Framework.exceptions import *
from GangaCore.Utility.logging import getLogger
import os, urllib, datetime, subprocess, time, fnmatch, shutil
#from GangaCore.Utility.Config import getConfig
import GangaCore.Utility.Config
#from GangaTest.Lib.TestRobot.Utils import Emailer

logger = getLogger()


class Checker(IAction):

    """ Checker implementation

    is setup to check a specified site for a new prerelease
    every 10 minutes.
    
    """

    #class variables
#    LastModifiedTime = ''
#    LastModified = -1
#    DownloadURL = ''
#    LastRelease = ''
#    cfgfile = ''
#    configfilename = 'JointCFG.ini'
#    NewRelease = False

    def execute(self, runid):
        #Get configuration data
        config = GangaCore.Utility.Config.getConfig('TestRobot')
        self._getconfig()
        #Email on Startup
        if (self.EmailOnStartup == True):
            subject = "Ganga - TestRobot automatic restart by crontab"
            text = "GangaTestRobot restarted on: %s" %(datetime.datetime.now().strftime("%H:%M:%S %d %b %Y"))
            self._emailer(subject, text)
        else:
            pass
        self.NewRelease = False
        while not self.NewRelease:
            self._heartbeat()
            #Then check for update
            CurrentTime = datetime.datetime.now()
            SleepDelta = datetime.timedelta(seconds=int(self.SleepTime))
            if (CurrentTime - self.LastCheckedTime) < SleepDelta:
                logger.debug("Last check was too recent")
                pass
            else:
                logger.debug("Checking for update...")
                self.NewRelease = self._checkfornewrelease()
                self.LastCheckedTime = datetime.datetime.now()
                config.setSessionValue('LastCheckedTime',self.LastCheckedTime.strftime("%d %b %Y %H:%M:%S"))
            if self.NewRelease:
                config.setSessionValue('VersionNumber',self.VersionNumber)
                config.setSessionValue('VersionTime',self.VersionTime.strftime("%d %b %Y %H:%M:%S"))
                break
            logger.debug("No update available, sleeping for %s" %(self.SleepTime))
            time.sleep(int(self.SleepTime))
            #write heartbeat
            self.NewRelease = False # Reset NewRelease flag

    def _getconfig(self):
       
        config = GangaCore.Utility.Config.getConfig('TestRobot')
        self.DownloadURL = config['ReleasePath']
        self.SleepTime = config['SleepTime']
        self.EmailOnStartup = config['EmailOnStartup']
        self.LastCheckedTime = config['LastCheckedTime']
        if ( self.LastCheckedTime == '0' ): 
            self.LastCheckedTime = self.InitialTime
        else:
            self.LastCheckedTime = datetime.datetime(*(time.strptime(self.LastCheckedTime,"%d %b %Y %H:%M:%S")[0:6]))
        # Get data from temp files
        self._getfiledata()

    def _getfiledata(self):
        gangaconfig = GangaCore.Utility.Config.getConfig('Configuration')
        gangadirname = gangaconfig['gangadir']
        self.LastVersionData = 'None'
        #for file in os.listdir(gangadirname):
        #    if fnmatch.fnmatch(file, 'LastVersionFile.txt'):
        filename = gangadirname+os.sep+"LastVersionFile.txt"
        try:
            f = open(filename)
            self.LastVersionData = f.readline()
            f.close()
        except IOError as e:
            logger.info(e)
            pass
        #dirs below will have been deleted on clean up
        config = GangaCore.Utility.Config.getConfig('TestRobot')
        #Set Install path with new directory  
        dirname = os.path.join(gangadirname, "Releases")
        try:
            os.mkdir(dirname)
        except OSError as e:
            logger.info(e)
        config.setSessionValue('InstallPath',dirname)
        #Sets job path
        dirname = os.path.join(gangadirname,'Jobs')
        try:
            os.mkdir(dirname)
        except OSError as e:
            logger.info(e)
        config.setSessionValue('JobDir',dirname)
        #return last version data
            
        
    def _checkfornewrelease(self):
    
           #obtain file
        try:
            f, inf = urllib.urlretrieve(self.DownloadURL+"/VERSIONS.txt")
        except:
            msg = "Failed to retrieve VERSIONS.txt from %s" % (self.DownloadURL)
            logger.error(msg)
            raise GangaRobotBreakError(msg,IOError)
        label = 'Content-Type'
        type = 'text/plain'
        for h in inf.headers:
            if (h.find(label) != -1):
                if (h.find(type) != -1):
                    # Found plain text file - this is what we want
                    pass
                else:
                    # Found other file - wrong place raise error
                    msg = "File not found or of wrong type at '%s'" % (self.DownloadURL+"VERSIONS")
                    raise GangaRobotBreakError(None,msg)
        #read lines into list
        f = open(f)
        VersionList = [v for v in f.readlines() if len(v)>2]
        logger.debug(VersionList)
        #find matching point
        #else return test first new
        i = -1
        match = -1
        for data in VersionList:
            i += 1
            if (data == self.LastVersionData) :
                match = i
                break
        #if none found, return first in list
        if (match == -1):
            self.VersionData = VersionList[0]
            self.VersionNumber = self._getversioninfo(self.VersionData)[0]
            self.VersionTime = self._getversioninfo(self.VersionData)[1]
            logger.debug("No matches found. Testing first in list")
            return True
        elif (match == (len(VersionList)-1)):
            self.VersionData = VersionList[(i-1)]
            logger.debug("No new releases found")
            #self.LastVersionData = ''
            return False
        # Assign next element as new data
        self.VersionData = VersionList[i+1]
        self.VersionNumber = self._getversioninfo(self.VersionData)[0]
        #self.VersionTime = self._getversioninfo(self.VersionData)[1]
        #strip list to current and new
        VersionList = VersionList[(i+1):]
        #while list size != 1
        while (len(VersionList) > 1):
            # scan through list
            for data in VersionList:
                # compare with current version
                dataNumber = self._getversioninfo(data)[0]
                if (dataNumber == self.VersionNumber):
                    # if so, strip list
                    VersionList = VersionList[1:]
                    break
            # assign list[0] to data
            self.VersionData = VersionList[0]
        self.VersionNumber = self._getversioninfo(self.VersionData)[0]
        self.VersionTime = self._getversioninfo(self.VersionData)[1]
        logger.info("Installing release %s created on %s" % (self.VersionNumber,self.VersionTime ))
        f.close()
        return True

    def _getversioninfo(self, VersionData):
        EqualsPoint = 0
        for i in range(len(VersionData)):
            if VersionData[i] == "-":
                EqualsPoint = i
        Number = VersionData[:(EqualsPoint-1)].strip()
        Date = VersionData[(EqualsPoint+1):].strip()
        try:
            Date = time.strptime(Date,"%d %b %Y %H:%M:%S")
        except ValueError as e:
           raise GangaRobotFatalError(" Incorrect format of file", ValueError)
        Date = datetime.datetime(*(Date[0:6]))
        Data = [Number, Date]
        return Data
                
    def _heartbeat(self):
        config = GangaCore.Utility.Config.getConfig('Configuration')
        heartbeatfile = str(config['gangadir'])+os.sep+"heartbeat.txt"
        heartbeattmpfile = str(config['gangadir'])+os.sep+"heartbeattemp.txt"
        try:
            f = open(heartbeatfile,'r')
            HeartBeatData = (f.read())[:-7].strip()
            f.close()
            HBtime = HeartBeatData[:15].strip()
            HeartBeatTime = datetime.datetime(*(time.strptime(HBtime,"%H:%M:%S %j %y")[0:6]))
            if ((datetime.datetime.now() - HeartBeatTime) > datetime.timedelta(minutes=10)):
                f = open(heartbeattmpfile, 'w')
                f.write(datetime.datetime.now().strftime("%H:%M:%S %j %y")+" - "+str(os.getpid()))
                f.close()
                try:
                    shutil.move(heartbeattmpfile,heartbeatfile)
                except:
                    raise GangaRobotContinueError("Failed to move heartbeat file")            
        except IOError as e:
            f  = open(heartbeatfile, 'w')
            f.write(datetime.datetime.now().strftime("%H:%M:%S %j %y")+" - "+str(os.getpid()))
            f.close()
            
    def _emailer(self, subject, text):
        #Function to email list on startup of ganga (following restart by cronjob)
        from email.MIMEText import MIMEText
        from smtplib import SMTP
        emailcfg = GangaCore.Utility.Config.getConfig('Robot')
        host = emailcfg['FileEmailer_Host']
        from_ = emailcfg['FileEmailer_From']
        recipients = [recepient.strip() for recepient in \
                        emailcfg['FileEmailer_Recipients'].split(',') \
                        if recepient.strip()]
        subject = "Ganga - TestRobot automatic restart by crontab"
        
        if not recipients:
            logger.warn('No recpients specified - email will not be sent')
            return
            
        logger.info("emailing files to %s." %(recipients))
        text = "GangaTestRobot restarted on: %s" %(datetime.datetime.now().strftime("%H:%M:%S %d %b %Y"))
        msg = MIMEText(text)
        msg['Subject'] = subject
        msg['From'] = from_
        msg['To'] = ', '.join(recipients)
        
        #send message
        session = SMTP()
        try:
            session.connect(host)
            session.sendmail(from_, recipients, msg.as_string())
            session.quit()
        except:
            logger.error("Failed to send notification of start-up")
        
        session.close()
            

    def __init__(self):
        self.InitialTime = datetime.datetime.fromtimestamp(0.0) #start of epoch
        #logger.info(self.LastCheckedTime)



import os
#import md5
import re
import sys
import socket

from threading import Thread, Lock
from Queue import Queue, Empty

from dq2.info import TiersOfATLAS
from dq2.common.DQException import *

try:
    import hashlib
    md = hashlib.md5()
except ImportError:
    # for Python << 2.5
    import md5
    md = md5.new()

## Utility functions ##
def get_md5sum(fname):
    ''' Calculates the MD5 checksum of a file '''

    f = open(fname, 'rb')
    #m = md.new()
    while True:
        d = f.read(8096)
        if not d:
            break
        md.update(d)
    f.close()
    return md.hexdigest()

def urisplit(uri):
   """
   Basic URI Parser according to STD66 aka RFC3986

   >>> urisplit("scheme://authority/path?query#fragment")
   ('scheme', 'authority', 'path', 'query', 'fragment')

   """
   # regex straight from STD 66 section B
   regex = '^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?'
   p = re.match(regex, uri).groups()
   scheme, authority, path, query, fragment = p[1], p[3], p[4], p[6], p[8]
   #if not path: path = None
   return (scheme, authority, path, query, fragment)

def get_srm_endpoint(dq2_site_id):
    '''
    Gets the SRM endpoint of a site registered in TiersOfATLAS.

    @param dq2_site_id is a DQ2 site id
    @return a dictionary containing the srm endpoint information
    '''

    srm_endpoint_info = {'token':None, 'endpt':None, 'se_host':None, 'se_path':None}
    re_srm2 = re.compile('^token:(.*):(srm:\/\/.*)\s*$')

    srm_endpt = TiersOfATLAS.getSiteProperty(dq2_site_id,'srm')

    if srm_endpt:
        mat = re_srm2.match(srm_endpt)
        if mat:
            # this is a SRMv2 endpoint specification
            srm_endpoint_info['token'] = mat.group(1)
            srm_endpoint_info['endpt'] = mat.group(2)

            endpt_data = urisplit(srm_endpoint_info['endpt'])
            srm_endpoint_info['se_host'] = endpt_data[1].split(':')[0]
            srm_endpoint_info['se_path'] = endpt_data[3].replace('SFN=','')
        else:
            # this is a SRMv1 endpoint specification
            srm_endpoint_info['token'] = None
            srm_endpoint_info['endpt'] = srm_endpt

            endpt_data = urisplit(srm_endpoint_info['endpt'])
            srm_endpoint_info['se_host'] = endpt_data[1].split(':')[0]
            srm_endpoint_info['se_path'] = endpt_data[2]

    return srm_endpoint_info

def get_srm_host(site):
    '''
    Gets the SRM hostname of the given site.
    '''
    srm_endpoint_info = get_srm_endpoint(site)

    return srm_endpoint_info['se_host']

def get_lfc_host(dq2_site_id):
    '''
    Gets the LFC host of a site registered in TiersOfATLAS.
    '''

    lfc_url = TiersOfATLAS.getLocalCatalog(dq2_site_id)
    if lfc_url:
        return lfc_url.split('/')[2][:-1]
    else:
        return None

def get_se_hostname(sename_replacements={}):
    '''
    Tries to determine the hostname of the ATLAS local SE by parsing $VO_ATLAS_DEFAULT_SE.

    The "sename_replacements" dictionary then is applied by replacing the detected sename with
    a right one given by human knowledge. The dictionary is given in the format:

        sename_replacements = { 'old(wrong)_sename_1': 'new(right)_sename_1',
                                'old(wrong)_sename_2': 'new(right)_sename_2',
                                ... ... }

    '''
    sename = ''
    if 'VO_ATLAS_DEFAULT_SE' in os.environ:
        sename  = os.environ['VO_ATLAS_DEFAULT_SE']

    if sename:
        # apply exceptions
        for old,new in sename_replacements.items():
            if sename == old:
                sename = new

    return sename

def get_transfer_protocols(sename=None):
    '''
    Tries to detect the data transfer protocols supported by the specified storage element.
    '''

    backup_bdii = 'lcg-bdii.cern.ch:2170'

    if not sename:
        sename = get_se_hostname()

    outputs = []
    protocols = []

    ## query to default BDII (LCG_GFAL_INFOSYS)
    if 'LCG_GFAL_INFOSYS' in os.environ:
        cmd = 'lcg-info --list-se --vo atlas --query SE=\'%s\' --attr Protocol --sed' % sename

        f = os.popen(cmd)
        outputs = map(lambda x:x.strip(), f.readlines())
        rc = f.close()

        print >> sys.stdout, 'resolving SE protocols with default BDII: %s | %s | %s' % (os.environ['LCG_GFAL_INFOSYS'], cmd, outputs)
    else:
        print >> sys.stdout, 'no default BDII defined by LCG_GFAL_INFOSYS'

    ## query to the backup BDII at CERN
    if not outputs:
        cmd = 'lcg-info --list-se --vo atlas --bdii \'%s\' --query SE=\'%s\' --attr Protocol --sed' % (backup_bdii, sename)
        f = os.popen(cmd)
        outputs = map(lambda x:x.strip(), f.readlines())
        rc = f.close()


        print >> sys.stdout, 'resolving SE protocols with backup BDII: %s | %s | %s' % (os.environ['LCG_GFAL_INFOSYS'], cmd, outputs)

    ## parsing the query results
    if not rc:
        re_se = re.compile('^%s' % sename.lower())

        for line in outputs:
            if re_se.match(line.lower()):
                se, protocol_str = line.split('%')
                if protocol_str:
                    protocols = protocol_str.split('&')
                break

    if protocols:
        ## keep unique protocol names and remove '_UNDEF_' in the list
        protocols = list( set( protocols ) )
        try:
            protocols.remove('_UNDEF_')
            print >> sys.stdout, 'protocol undefined for SE: %s' % sename
        except ValueError:
            pass

    return protocols

def get_site_domain(domain_replacements={}):
    '''
    Tries to determine the site domain from varies approaches:
     - parse EDG_WL_RB_BROKERINFO: available if job is brokered by EDG RB
     - parse GLITE_WMS_RB_BROKERINFO: available if job is brokered by GLITE WMS
     - parse GANGA_LCG_CE: available if job's CE is assigned through Ganga
     - parse VO_ATLAS_DEFAULT_SE: assuming the default SE of ATLAS is sitting inside the same domain
     - parse localhost domain in the worst case

    The "domain_replacements" dictionary then is applied by replacing the detected domain with
    a right one given by human knowledge. The dictionary is given in the format:

        domain_replacementss = { 'old(wrong)_domain_1': 'new(right)_domain_1',
                                 'old(wrong)_domain_2': 'new(right)_domain_2',
                                 ... ... }
    '''

    site_domain = None

    hostname = None

    # First choice: EDG_WL_RB_BROKERINFO or GLITE_WMS_RB_BROKERINFO
    if 'EDG_WL_RB_BROKERINFO' in os.environ:
        try:
            f = open(os.environ['EDG_WL_RB_BROKERINFO'], "r")
            lines = f.readlines()
            for line in lines:
                match = re.search('name = "(\S*):2119', line)
                if match:
                    hostname =  [ match.group(1) ]
        except:
            pass

    if 'GLITE_WMS_RB_BROKERINFO' in os.environ:
        try:
            f = open(os.environ['GLITE_WMS_RB_BROKERINFO'], "r")
            lines = f.readlines()
            for line in lines:
                match = re.search('name = "(\S*):2119', line)
                if match:
                    hostname =  [ match.group(1) ]
        except:
            pass

    # Second choice: GANGA_LCG_CE
    if not hostname and 'GANGA_LCG_CE' in os.environ:
        try:
            hostname = re.findall('(\S*):2119',os.environ['GANGA_LCG_CE'])
            #print hostname, lcgcename
        except:
            pass

    # Third choice: VO_ATLAS_DEFAULT_SE
    if not hostname and 'VO_ATLAS_DEFAULT_SE' in os.environ:
        hostname = os.environ['VO_ATLAS_DEFAULT_SE']

    # Fourth choice: local hostname
    if not hostname:
        hostname = socket.gethostbyaddr(socket.gethostname())

    if hostname.__class__.__name__=='list' or hostname.__class__.__name__=='tuple':
        hostname = hostname[0]

    if hostname:
        site_domain = re.sub('^[\w\-]+\.','',hostname)

        # apply exceptions
        for old,new in domain_replacements.items():
            if site_domain == old:
                site_domain = new

    return site_domain

def resolve_dq2_local_site_id(ds_locations, site_domain, se_hostname, force_siteid_domain={}, force_siteid_se={}):
    '''
    resolves the DQ2_LOCAL_SITE_ID based on 3 factors:
     - the dataset location list
     - the site domain
     - the default SE host
     - the dictionaries containing the local_site_id enforcement

    The "force_siteid_domain" gives a list in which the matched site_domain (regx match)
    will be limited to the given list of DQ2 site ids.

    The "force_siteid_se" gives a list in which the matched se_hostname (regx match) 
    will be limited to the given list of DQ2 site ids.

    it returns a final DQ2 site id which is local and containing the dataset replica
    or an empty string if there is no matched DQ2 site id.
    '''
    dq2_local_site_id = ''

    toa_check = True

    # get dq2 site ids matching the site_domain and se_hostname
    dq2_local_ids = []

    # checking if the given site_domain in the enforcement dictionary
    if force_siteid_domain:
        for mydomain, mysites in force_siteid_domain.items():
            re_domain = re.compile(mydomain)
            if re_domain.match(site_domain):
                dq2_local_ids = mysites
                toa_check = False
                break

    # checking if the given se_hostname in the enforcement dictionary
    if force_siteid_se:
        for myse, mysites in force_siteid_se.items():
            re_se = re.compile(myse)
            if re_se.match(se_hostname):
                dq2_local_ids = mysites
                toa_check = False
                break

    # if enforcement is never applied, do the detection for dq2_local_ids via ToA
    if toa_check:
        
        alternateNameDict    = {}
        
        for sitename in TiersOfATLAS.getAllSources():

            # compose the altname:siteid dictionary
            altnames = TiersOfATLAS.getSiteProperty(sitename,'alternateName')
            if altnames:
                for altname in altnames:
                    if altname not in alternateNameDict:
                        alternateNameDict[altname] = []
                    alternateNameDict[altname].append(sitename)
            
            # First search for srm
            dq2srm = TiersOfATLAS.getSiteProperty(sitename,'srm')
            if dq2srm and dq2srm.find(se_hostname)>=0:
                dq2_local_ids.append(sitename)

            # Second search for domainname
            dq2domain = TiersOfATLAS.getSiteProperty(sitename,'domain')
            if dq2domain and dq2domain.find(site_domain)>=0:
                dq2_local_ids.append(sitename)


        # Thirdly search for sites with the same alternateName
        more_dq2_local_ids = []
        for sitename in dq2_local_ids:
            altnames = TiersOfATLAS.getSiteProperty(sitename,'alternateName')
            if altnames:
                for altname in altnames:
                    if altname in alternateNameDict:
                        more_dq2_local_ids += alternateNameDict[altname]

        dq2_local_ids += more_dq2_local_ids


    # resolving the best location according to the dataset locations
    #  - get the common part between dq2_local_ids and ds_locations
    #  - pick the first one in the common part
    candidates = list(set(dq2_local_ids) & set(ds_locations))

    print >> sys.stdout, str(candidates)

    if candidates:
        ## pick up the site whose corresponding se is matching the se_hostname
        for c in candidates:
            srm_info = get_srm_endpoint(c)
            if srm_info['se_host'] == se_hostname:
                dq2_local_site_id = c
                break
                
        ## otherwise, take the first candidate in the list
        if not dq2_local_site_id:
            dq2_local_site_id = candidates[0]

    return dq2_local_site_id

def make_FileStager_jobOption(pfns, gridcopy=True, protocol='lcgcp', maxEvent=-1, skipEvent = 0, fs_cp_cmd='', fs_cp_args=[], fs_of_prefix='file:', ic_jo_path='', fs_jo_path=''):
    '''
    creates the Athena job option file for FileStager.

    @param pfns specifies a list of files in physical paths
    @param gridcopy indicates if appending "gridcopy://" in front of each of pfns
    @param protocol indicates the preferred copy protocol
    @param maxEvent specifies the max. number of events
    @param skipEvent specifies the number of events to skip
    @param fs_cp_cmd specifies the exact copy command to be used by FileStager
    @param fs_cp_args specifies the exact copy command arguments to be used by FileStager
    @param fs_of_prefix specifies the prefix to be added in front of the destination file path in the copy command used by FileStager
    @param ic_jo_path specifies the path of the Athena job option stating the InputCollection configuration for Athena
    @param fs_jo_path specifies the path of the Athena job option stating the FileStager specific configurations
    '''

    jOptionFS = """#################################################################################################
# FileStager job option file generated by Ganga
#################################################################################################

import os

fsCopyCmd  = ###FSCOPYCMD###
fsCopyArgs = ###FSCOPYARGS###
fsOutputPrefix = ###FSOUTPUTPREFIX###

mySampleList = ###SAMPLELIST###

mySampleFile = 'sample.list'

myProtocol = ###GRIDCOPYPROTOCOL###

os.environ['FILE_STAGER_PROTOCOL'] = myProtocol

f = open(mySampleFile,'w')
for l in mySampleList:
    f.write(l + '\\n')
f.close()

## import filestager tool
from FileStager.FileStagerTool import FileStagerTool

## File with input collections
#stagetool = FileStagerTool(sampleList=mySampleList)
stagetool = FileStagerTool(sampleFile=mySampleFile)

## load the copy command wrapper if it exists on the WN
import os, os.path
if not fsCopyCmd:
    fsCopyCmd  = os.path.join( os.getcwd(), 'fs-copy.py' )
    fsCopyArgs = []
    fsOutputPrefix = 'file:'

stagetool.CpCommand = fsCopyCmd
stagetool.CpArguments = fsCopyArgs
stagetool.OutfilePrefix = fsOutputPrefix

print '*******'
print 'FS copy command   : %s' % stagetool.CpCommand
print 'FS copy arguments : %s' % stagetool.CpArguments
print 'FS dest. prefix   : %s' % stagetool.OutfilePrefix
print '*******'

## enforce storing copied files in current working directory
stagetool.tmpDir = os.getcwd()

#################################################################################################
# Configure the FileStager -- no need to change these lines
#################################################################################################

## get Reference to existing Athena job
from AthenaCommon.AlgSequence import AlgSequence
thejob = AlgSequence()

## check if collection names begin with "gridcopy"
print "doStaging?", stagetool.DoStaging()

## Import file stager algorithm
from FileStager.FileStagerConf import FileStagerAlg

## filestageralg needs to be the first algorithm added to the thejob.
if stagetool.DoStaging():
   thejob += FileStagerAlg('FileStager')
   thejob.FileStager.InputCollections = stagetool.GetSampleList()
   #thejob.FileStager.PipeLength = 2
   thejob.FileStager.VerboseStager = True
   thejob.FileStager.BaseTmpdir    = stagetool.GetTmpdir()
   thejob.FileStager.InfilePrefix  = stagetool.InfilePrefix
   thejob.FileStager.OutfilePrefix = stagetool.OutfilePrefix
   thejob.FileStager.CpCommand     = stagetool.CpCommand
   thejob.FileStager.CpArguments   = stagetool.CpArguments
   thejob.FileStager.FirstFileAlreadyStaged = stagetool.StageFirstFile

#################################################################################################
# Pass collection names to EventSelector
#################################################################################################

## set input collections
FileStagerInputCollection = []

if stagetool.DoStaging():
  FileStagerInputCollection = stagetool.GetStageCollections()
else:
  FileStagerInputCollection = stagetool.GetSampleList()

### Prepend AutoConfiguration (JE) #############################
try:
  from EventSelectorAthenaPool.EventSelectorAthenaPoolConf import EventSelectorAthenaPool
  orig_ESAP__getattribute =  EventSelectorAthenaPool.__getattribute__

  def _dummy(self,attr):
    if attr == 'InputCollections':
      return FileStagerInputCollection
    else:
      return orig_ESAP__getattribute(self,attr)

  EventSelectorAthenaPool.__getattribute__ = _dummy
  print 'Overwrite InputCollections'
  print EventSelectorAthenaPool.InputCollections
except:
  try:
    EventSelectorAthenaPool.__getattribute__ = orig_ESAP__getattribute
  except:
    pass
   
try:
  import AthenaCommon.AthenaCommonFlags

  def _dummyFilesInput(*argv):
    return FileStagerInputCollection

  AthenaCommon.AthenaCommonFlags.FilesInput.__call__ = _dummyFilesInput
except:
  pass

try:
  import AthenaCommon.AthenaCommonFlags

  def _dummyGet_Value(*argv):
    return FileStagerInputCollection

  for tmpAttr in dir (AthenaCommon.AthenaCommonFlags):
    import re
    if re.search('^(Pool|BS).*Input$',tmpAttr) is not None:
      try:
        getattr(AthenaCommon.AthenaCommonFlags,tmpAttr).get_Value = _dummyGet_Value
      except:
        pass
except:
  pass

try:
  from AthenaServices.SummarySvc import *
  useAthenaSummarySvc()
except:
  pass
"""

    jOptionInput = """
ic = []

if FileStagerInputCollection:
    ic = FileStagerInputCollection

## assume we're dealing with AODs, else ESDs
poolESDInput = False
if len(ic)>0:
    if ic[0].find('ESD')>0: poolESDInput = True

## get a handle on the ServiceManager
if os.environ.has_key('DATASETDATATYPE') and os.environ['DATASETDATATYPE']=='MuonCalibStream':
    svcMgr.MuonCalibStreamFileInputSvc.InputFiles = ic

elif not os.environ.has_key('RECEXTYPE') or os.environ['RECEXTYPE'] == '':
    ## default: EventSelector
    try:
        svcMgr = theApp.serviceMgr()
        svcMgr.EventSelector.InputCollections = ic
        #svcMgr.EventSelector.SkipBadFiles = True
    except Exception as inst:
        pass

    ## else: athenaCommonFlags
    if not poolESDInput:
        try:
            ## the Input AOD File(s)
            from AthenaCommon.AthenaCommonFlags import athenaCommonFlags
            athenaCommonFlags.FilesInput = ic
        except Exception as inst:
            pass
    else:
        try:
            ## the Input ESD File(s)
            from AthenaCommon.AthenaCommonFlags import athenaCommonFlags
            athenaCommonFlags.FilesInput = ic
        except Exception as inst:
            pass
else:
    try:
        print "Using RECEXTYPE in FILE_STAGER"
        from AthenaCommon.AthenaCommonFlags import athenaCommonFlags
        athenaCommonFlags.Pool###RECEXTYPE###Input.set_Value_and_Lock( ic )
        athenaCommonFlags.FilesInput.set_Value_and_Lock( ic )
        athenaCommonFlags.EvtMax.set_Value_and_Lock(###MAXEVENT###)
        athenaCommonFlags.SkipEvents.set_Value_and_Lock(###SKIPEVENT###)
    except:
        pass

## override the event number
theApp.EvtMax = ###MAXEVENT###
## override the  number of events to skip
ServiceMgr.EventSelector.SkipEvents = ###SKIPEVENT###
    
## setting for event picking
if os.environ.has_key('ATHENA_RUN_EVENTS'):
    from AthenaCommon.AlgSequence import AthSequencer
    seq = AthSequencer('AthFilterSeq')
    from GaudiSequencer.PyComps import PyEvtFilter
    seq += PyEvtFilter('alg',evt_info='',)
    revt = eval(os.environ['ATHENA_RUN_EVENTS'])
    run_evt = []
    for i in range(len(revt)):
        run_evt.append((revt[i][0], revt[i][1]))
    seq.alg.evt_list = run_evt
    seq.alg.filter_policy = str(os.environ['ATHENA_FILTER_POLICY'])
    for tmpStream in theApp._streams.getAllChildren():
        fullName = tmpStream.getFullName()
        if fullName.split('/')[0] == 'AthenaOutputStream':
             tmpStream.AcceptAlgs = [seq.alg.name()]

"""

    ick = False

    ## create option file for FileStager
    if gridcopy:
        pfns = map(lambda x:'gridcopy://'+x, pfns)

    jOptionFS = jOptionFS.replace('###FSCOPYCMD###', repr(fs_cp_cmd))
    jOptionFS = jOptionFS.replace('###FSCOPYARGS###', repr(fs_cp_args))
    jOptionFS = jOptionFS.replace('###FSOUTPUTPREFIX###', repr(fs_of_prefix))
    jOptionFS = jOptionFS.replace('###SAMPLELIST###', repr(pfns))
    jOptionFS = jOptionFS.replace('###GRIDCOPYPROTOCOL###', repr(protocol))
    if 'RECEXTYPE' in os.environ:
        jOptionInput = jOptionInput.replace('###RECEXTYPE###', os.environ['RECEXTYPE'])
    else:
        jOptionInput = jOptionInput.replace('###RECEXTYPE###', 'ESD')
    f = open( fs_jo_path, 'w' )
    f.write(jOptionFS)
    f.close()

    ## create option file for Inputs
    jOptionInput = jOptionInput.replace('###MAXEVENT###', repr(maxEvent))
    jOptionInput = jOptionInput.replace('###SKIPEVENT###', repr(skipEvent))
    f = open( ic_jo_path, 'w' )
    f.write(jOptionInput)
    f.close()

    ick = True

    # Write flatfile input.txt for ARA
    flatFile = 'input.txt'
    outFlatFile = open(flatFile,'w')
    for pfn in pfns:
       outFlatFile.write('%s\n' %pfn)
    outFlatFile.close()

    return ick

def get_pfns(lfc_host, guids, nthread=10, dummyOnly=False, debug=False):
    '''
    getting pfns and checksum type/value corresponding to the given list of files represented
    by guids.

    @param lfc_host specifies the host of the local file catalogue
    @param guids is a list of GUIDs
    @param nthread is the amount of parallel threads for querying the LFC, 10 by default
    @param dummyOnly indicates if this routine returns only the dummy registries, default is False
    @param debug indicates if debugging messages are printed

    @return a dictionary of PFNs in the following format:

        pfns = { guid_1: [replica_1_pfn, replica_2_pfn, ...],
                 guid_2: [replica_1_pfn, replica_2_pfn, ...],
                 ... }

        and a dictionary of checksum type/value in the following format:

        csum = { guid_1: {'csumtype': checksum_type, 'csumvalue': checksum_value},
                 guid_2: {'csumtype': checksum_type, 'csumvalue': checksum_value},
                 ... }

    It uses the LFC multi-thread library: lfcthr, each worker thread works
    on the query of 1000 LFC registries.

    If dummyOnly, then only the pfns doublely copied on the
    the same SE are presented (determinated by SE hostname parsed from
    the PFNs).
    '''

    print >> sys.stdout, 'resolving physical locations of replicas'

    try:
        import lfcthr
    except ImportError as exp:
        print >> sys.stderr, '%s' % str(exp)
        print >> sys.stderr, 'unable to load LFC python module. Please check LCG UI environment.'
        print >> sys.stderr, 'python path: %s' % repr(sys.path)
        return {}

    pfns = {}
    csum = {}

    # divide guids into chunks if the list is too large
    chunk_size = 1000
    num_chunks = len(guids) / chunk_size
    if len(guids) % chunk_size > 0:
        num_chunks += 1

    chunk_offsets = []
    for i in range(num_chunks):
        chunk_offsets.append(i*chunk_size)

    # backup the current LFC_HOST
    lfc_backup = None
    try:
        lfc_backup = os.environ['LFC_HOST']
    except:
        pass

    # set to use a proper LFC_HOST
    os.putenv('LFC_HOST', lfc_host)

    def _resolveDummy(_pfns):
        '''resolving the dummy PFNs based on SE hostname'''
        _pfns_dummy = {}
        for _guid in _pfns.keys():
            _replicas = _pfns[_guid]
            _replicas.sort()
            seCache  = None
            pfnCache = None
            id = -1
            for _pfn in _replicas:
                id += 1
                _se = urisplit(_pfn)[1]
                if _se != seCache:
                    seCache  = _se
                    pfnCache = _pfn
                else:
                    # keep the dummy PFN
                    if _guid not in _pfns_dummy:
                        _pfns_dummy[_guid] = [pfnCache]
                    _pfns_dummy[_guid].append(_pfn)
        return _pfns_dummy

    ## setup worker queue for LFC queries
    wq = Queue(len(chunk_offsets))
    for offset in chunk_offsets:
        wq.put(offset)

    mylock = Lock()
    def worker(id):
        # try to connect to LFC
        if lfcthr.lfc_startsess('', '') == 0:
            while not wq.empty():
                try:
                    idx_beg = wq.get(block=True, timeout=1)
                    idx_end = idx_beg + chunk_size
                    if idx_end > len(guids):
                        idx_end = len(guids)

                    result, list1 = lfcthr.lfc_getreplicas(guids[idx_beg:idx_end],"")

                    if len(list1) > 0:
                        ## fill up global pfns dictionary and global csum dictionary
                        mylock.acquire()
                        for s in list1:
                            if s is not None:
                                if s.sfn:
                                    if s.guid not in pfns:
                                        pfns[s.guid] = []
                                    pfns[s.guid].append(s.sfn)
                                    csum[s.guid] = {'csumtype':'', 'csumvalue':''}
                                if s.csumtype:
                                    csum[s.guid]['csumtype'] = s.csumtype
                                if s.csumvalue:
                                    csum[s.guid]['csumvalue'] = s.csumvalue
                        mylock.release()
                except Empty:
                    pass
            # close the LFC session
            lfcthr.lfc_endsess()
        else:
            print >> sys.stderr, 'cannot connect to LFC'

    # initialize lfcthr
    lfcthr.init()

    # prepare and run the query threads
    threads = []
    for i in range(nthread):
        t = Thread(target=worker, kwargs={'id': i})
        t.setDaemon(False)
        threads.append(t)

    for t in threads:
        t.start()

    for t in threads:
        t.join()

    if dummyOnly:
        pfns = _resolveDummy(pfns)

    # roll back to the original LFC_HOST setup in the environment
    if lfc_backup:
        os.putenv('LFC_HOST', lfc_host)

    return pfns, csum

def get_srmv2_sites(cloud=None, token=None, debug=False):
    '''
    Gets a list of SRMV2 enabled DDM sites in a given cloud.

    @param cloud is the ATLAS cloud name
    @param token restricts the output to only certain srmv2 tokens
    @param debug indicates if debugging messages are printed

    @return a list of ATLAS srmv2-enabled site names

    if token is given, only the site with the specific token type
    will be selected.
    '''

    srmv2_sites = []

    ## a better way of getting all sites within a cloud
    ## however, it seems there is a bug in DQ2 API so it
    ## always returns an empty site list.
    # all_sites   = TiersOfATLAS.getSitesInCloud(cloud)

    ## a bit of hack with non-public DQ2 API interface
    cache = TiersOfATLAS.ToACache

    all_sites = []
    if not cloud:
        all_sites = TiersOfATLAS.getAllDestinationSites()
    else:
        if cloud == 'T0':
            return ['CERNPROD']
        if cloud not in cache.dbcloud:
            return []
        all_sites = TiersOfATLAS.getSites(cache.dbcloud[cloud])

    for site in all_sites:
        srm = TiersOfATLAS.getSiteProperty(site,'srm')

        # presuming the srm endpoint looks like:
        #   token:ATLASDATADISK:srm://grid-cert-03.roma1.infn.it ...
        if srm is not None and srm.find('token') != -1:
            if token:
                if srm.split(':')[1] == token:
                    srmv2_sites.append(site)
            else:
                srmv2_sites.append(site)

    return srmv2_sites



'''General information and conventions about job data management 

On the worker node these environment variables have been already defined.
The user executable should take into account these information to run correctly.

WN_INPUTFILES : points to the directory where input files are downloaded.

WN_OUTPUTFILES : points to directory where you can put your output files to be 
registered on grid. E.g. the user bash script should contains the following 
command in FastSim production:
mv ./${PRODSERIES}/FastSim/${PRODSCRIPT}/${DG}/${GENERATOR}/${RUNNUM}/* ${WN_OUTPUTFILES}/

WN_INPUTLIST : points to a file in the WN_INPUTFILES directory which contains
 the input files absolute path list.

Stage out operations are the same independently by use case
Each file in WN_OUTPUTFILES is registered in:

A) grid:
- /grid/superbvo.org/analysis/
- srm://storm-fe-superb.cr.cnaf.infn.it/sb_analysis/
- /storage/gpfs_superb/sb_analysis/ 
+ <user_identity_from_certificate>/<date_idjob_usertaballsenzaestensione_jname>/output/subjobid_nomefile
B) User can register or not output files as dataset into dataplacement DB using the class SBOutputDataset
C) The outputsandbox is used in the case of small files that should not be registered in grid
'''



##########################################################################
# Ganga Project. http://cern.ch/ganga
##########################################################################

from GangaCore.GPIDev.Adapters.IGangaFile import IGangaFile
from GangaCore.GPIDev.Adapters.IPrepareApp import IPrepareApp
from GangaCore.GPIDev.Adapters.IRuntimeHandler import IRuntimeHandler
from GangaCore.GPIDev.Schema import Schema, Version, SimpleItem, ComponentItem, GangaFileItem

from GangaCore.Utility.Config import getConfig

from GangaCore.GPIDev.Lib.File import File, ShareDir
from GangaCore.Core.exceptions import ApplicationConfigurationError, ApplicationPrepareError

from GangaCore.Utility.logging import getLogger

from GangaCore.GPIDev.Base.Proxy import getName, isType, stripProxy

import os
import shutil
from GangaCore.Utility.files import expandfilename

from GangaDirac.Lib.Files.DiracFile import DiracFile
from GangaCore.GPIDev.Lib.File import LocalFile

logger = getLogger()

class Im3ShapeApp(IPrepareApp):

    """
    This App is to store the configuration of the Im3Shape app which is to be run according to a given set of configs:

    i.e.
    ./run_dir/exe_name <someData> ini_location catalog <someOutput> rank size

    e.g.
    ./run_dir/run-im3shape someData.fz ini_file.ini all someData.fz.0.20 0 20

    The input and output file names are configured in the RTHandler based upon the inputdata given to a particular job.

    The im3_location is the path to a .tgz file (or some file which can be extracted by the RTHandler) which gives the im3shape-grid (run_dir) folder containing the run-im3shape app (exe-name) on the WN
    """
    _schema = Schema(Version(1, 0), {
        ## Required to configure Im3ShapeApp
        'im3_location': GangaFileItem(defvalue=None, doc="Location of the Im3Shape program tarball"),
        'exe_name': SimpleItem(defvalue='run-im3shape', doc="Name of the im3shape binary"),
        'ini_location': GangaFileItem(defvalue=None, doc=".ini file used to configure Im3Shape"),
        'blacklist': GangaFileItem(defvalue=None, doc="Blacklist file for running Im3Shape"),
        'rank': SimpleItem(defvalue=0, doc="Rank in the split of the tile from splitting"),
        'size': SimpleItem(defvalue=200, doc="Size of the splitting of the tile from splitting"),
        'catalog': SimpleItem(defvalue='all', types=[str], doc="Catalog which is used to describe what is processed"),
        'run_dir': SimpleItem(defvalue='im3shape-grid', types=[str], doc="Directory on the WN where the binary is"),
        ## Below is needed for prepared state stuff
        'is_prepared': SimpleItem(defvalue=None, strict_sequence=0, visitable=1, copyable=1, hidden=0, typelist=[None, ShareDir], protected=0, comparable=1, doc='Location of shared resources. Presence of this attribute implies the application has been prepared.'),
        'hash': SimpleItem(defvalue=None, typelist=[None, str], hidden=0, doc='MD5 hash of the string representation of applications preparable attributes'),
    })
    _category = 'applications'
    _name = 'Im3ShapeApp'
    _exportmethods = ['prepare', 'unprepare']

    def unprepare(self, force=False):
        """
        Revert an Im3ShapeApp application back to it's unprepared state.
        args:
            force (bool): should force the unprepare step to run
        """
        logger.debug('Running unprepare in Im3ShapeApp')
        if self.is_prepared is not None:
            self.decrementShareCounter(self.is_prepared)
            self.is_prepared = None
        self.hash = None

    def prepare(self, force=False):
        """
        This prepares the Im3ShapeApp application and copies any LocalFile objects which are allocated to: im3_location, ini_location and blacklist into the prepared sandbox to be shipped to the WN
        Args:
            force (bool): Should force the prepare step to run
        """

        if (self.is_prepared is not None) and (force is not True):
            raise ApplicationPrepareError('%s application has already been prepared. Use prepare(force=True) to prepare again.' % getName(self))

        logger.info('Preparing %s application.' % getName(self))
        self.is_prepared = ShareDir()
        logger.info('Created shared directory: %s' % (self.is_prepared.name))

        try:
            # copy any 'preparable' objects into the shared directory
            send_to_sharedir = self.copyPreparables()
            # add the newly created shared directory into the metadata system
            # if the app is associated with a persisted object
            self.checkPreparedHasParent(self)
            for file_ in [self.ini_location, self.im3_location, self.blacklist]:
                if isinstance(file_, LocalFile):
                    self.copyIntoPrepDir(file_.namePattern)
                assert type(file_) in [LocalFile, DiracFile]
            # return
            # [os.path.join(self.is_prepared.name,os.path.basename(send_to_sharedir))]
            self.post_prepare()

        except Exception as err:
            logger.debug("Err: %s" % str(err))
            self.unprepare()
            raise

        return 1

    def configure(self, masterappconfig):
        """
        This is a null-op effecitvely, we may add something here in the future but this function is stub
        This is required so that the job will submit
        """
        return (None, None)




################################################################################
# GangaND280 Project.
# Anthony Hillairet
# Created  5/02/2015
################################################################################
"""@package TRExPlusOAAnalysis
This module is designed to run TREx followed by oaAnalysis
"""

from GangaCore.GPIDev.Adapters.IPrepareApp import IPrepareApp
from GangaCore.GPIDev.Adapters.IPrepareApp import IPrepareApp
from GangaCore.GPIDev.Adapters.IRuntimeHandler import IRuntimeHandler
from GangaCore.GPIDev.Schema import *

from GangaCore.Utility.Config import getConfig

from GangaCore.GPIDev.Lib.File import *
from GangaCore.GPIDev.Lib.Registry.PrepRegistry import ShareRef
from GangaCore.GPIDev.Base.Proxy import isType
from GangaCore.Core.exceptions import ApplicationConfigurationError

import os, shutil, commands, re, time
from GangaCore.Utility.files import expandfilename
shared_path = os.path.join(expandfilename(getConfig('Configuration')['gangadir']),'shared',getConfig('Configuration')['user'])

class TRExPlusOAAnalysis(IPrepareApp):
    """
    TRExPlusOAAnalysis application running any ND280 executables.

    The required input for this module are:
        app.filenamesubstr = 'myFantasticVFT.root'
        app.cmtsetup = '/home/me/myT2KWork/setup.sh'

    NOTE: Ganga will run a bash script so the CMT setup script must be in sh/bash.

    It is also possible to pass command line arguments to the executable.
    For example to analyze only 10 events:
        app.trex_args = ['-n',10]
    """
    _schema = Schema(Version(1,1), {
        'cmtsetup' : SimpleItem(defvalue=None,doc='Setup script in bash to set up cmt and the cmt package of the executable.', typelist=['str','type(None)']),
        'trex_args' : SimpleItem(defvalue=[],typelist=['str','GangaCore.GPIDev.Lib.File.File.File','int'],sequence=1,strict_sequence=0,doc="List of arguments for the executable. Arguments may be strings, numerics or File objects."),
        'oaana_only' : SimpleItem(defvalue=False,doc='Run only oaAnalysis on the input files, i.e. skip the reconstruction.', typelist=['bool']),
        'oaana_args' : SimpleItem(defvalue=[],typelist=['str','GangaCore.GPIDev.Lib.File.File.File','int'],sequence=1,strict_sequence=0,doc="List of arguments for the executable. Arguments may be strings, numerics or File objects."),
        'filenamesubstr' : SimpleItem(defvalue=None,doc='This string will be substituted by "trex" in the TREx output filename and "anal" in the oaAnalysis output filename.', typelist=['str','type(None)']),
        'env' : SimpleItem(defvalue={},typelist=['str'],doc='Environment'),
        } )
    _category = 'applications'
    _name = 'TRExPlusOAAnalysis'
    _scriptname = None
    _exportmethods = []
    _GUIPrefs = [ { 'attribute' : 'trex_args', 'widget' : 'String_List' },
                  { 'attribute' : 'oaana_args', 'widget' : 'String_List' },
                  { 'attribute' : 'filenamesubstr', 'widget' : 'String' },
                  { 'attribute' : 'env', 'widget' : 'DictOfString' } ]

    _GUIAdvancedPrefs = [ { 'attribute' : 'trex_args', 'widget' : 'String_List' },
                          { 'attribute' : 'oaana_args', 'widget' : 'String_List' },
                          { 'attribute' : 'filenamesubstr', 'widget' : 'String' },
                          { 'attribute' : 'env', 'widget' : 'DictOfString' } ]

    def __init__(self):
        super(TRExPlusOAAnalysis,self).__init__()


    def configure(self,masterappconfig):
        if self.cmtsetup is None:
          raise ApplicationConfigurationError('No cmt setup script given.')

        # __________ TREx first ____________
        trex_args = convertIntToStringArgs(self.trex_args)

        job = self.getJobObject()

        # Need to handle the possibility of multiple output files !
        # setup the output file
        for arg in trex_args:
          if arg == '-o':
            raise ApplicationConfigurationError('Option "-o" given in trex_args. The module will define the output filename.')

        # So get the list of filenames get_dataset_filenames() and create a file containing the list of files and put it in the sandbox
        if job.inputdata is None:
          raise ApplicationConfigurationError('The inputdata variable is not defined.')
        fileList = job.inputdata.get_dataset_filenames()
        if len(fileList) < 1:
          raise ApplicationConfigurationError('No input data file given.')
        trex_args.extend(fileList)

        firstFile = fileList[0].split('/')[-1]
        # Define the output
        trex_args.append('-o')
        if self.filenamesubstr is None:
          trex_outputfile = 'recoOutput.root'
        else:
          trex_outputfile = firstFile.replace(self.filenamesubstr, "trex")

        trex_args.append(trex_outputfile)

        # __________ Now oaAnalysis ____________
        oaana_args = convertIntToStringArgs(self.oaana_args)

        job = self.getJobObject()

        # Need to handle the possibility of multiple output files !
        # setup the output file
        for arg in oaana_args:
          if arg == '-o':
            raise ApplicationConfigurationError('Option "-o" given in oaana_args. You must use the oaana_outputfile variable instead.')

        oaana_args.append('-o')
        if self.filenamesubstr is None:
          oaana_outputfile = 'recoOutput.root'
        else:
          oaana_outputfile = firstFile.replace(self.filenamesubstr, "anal")
          # protection against failed substitution
          if oaana_outputfile == trex_outputfile:
            oaana_outputfile = oaana_outputfile.replace(".root","_anal.root")
        oaana_args.append(oaana_outputfile)

        # Use the reco output as an input for the VFT processing.
        if self.oaana_only:
            oaana_args.extend(fileList)
        else:
            oaana_args.append(trex_outputfile)

        trex_argsStr = ' '.join(trex_args)
        oaana_argsStr = ' '.join(oaana_args)
        # Create the bash script and put it in input dir.
        script = '#!/bin/bash\n'
        script += 'source '+self.cmtsetup+'\n'
        if not self.oaana_only:
            script += 'RunTREx.exe '+trex_argsStr+'\n'
        script += 'RunOAAnalysis.exe '+oaana_argsStr+'\n'

        from GangaCore.GPIDev.Lib.File import FileBuffer

        scriptname = 'TRExPlusOAAnalysis.sh'
        job.getInputWorkspace().writefile(FileBuffer(scriptname,script),executable=1)

        self._scriptname = job.inputdir+scriptname

        return (None,None)



config = getConfig('defaults_TRExPlusOAAnalysis') #_Properties


def convertIntToStringArgs(args):

    result = []
    
    for arg in args:
        if isinstance(arg,int):
            result.append(str(arg))
        else:
            result.append(arg)

    return result



class RTHandler(IRuntimeHandler):
    def prepare(self,app,appconfig,appmasterconfig,jobmasterconfig):
        from GangaCore.GPIDev.Adapters.StandardJobConfig import StandardJobConfig

        return StandardJobConfig(app._scriptname,app._getParent().inputsandbox,[],app._getParent().outputsandbox,app.env)
        

class LCGRTHandler(IRuntimeHandler):
    def prepare(self,app,appconfig,appmasterconfig,jobmasterconfig):
        from GangaCore.Lib.LCG import LCGJobConfig

        return LCGJobConfig(app._scriptname,app._getParent().inputsandbox,[],app._getParent().outputsandbox,app.env)

class gLiteRTHandler(IRuntimeHandler):
    def prepare(self,app,appconfig,appmasterconfig,jobmasterconfig):
        from GangaCore.Lib.gLite import gLiteJobConfig

        return gLiteJobConfig(app._scriptname,app._getParent().inputsandbox,[],app._getParent().outputsandbox,app.env)

from GangaCore.GPIDev.Adapters.ApplicationRuntimeHandlers import allHandlers

allHandlers.add('TRExPlusOAAnalysis','LSF', RTHandler)
allHandlers.add('TRExPlusOAAnalysis','Local', RTHandler)
allHandlers.add('TRExPlusOAAnalysis','PBS', RTHandler)
allHandlers.add('TRExPlusOAAnalysis','SGE', RTHandler)
allHandlers.add('TRExPlusOAAnalysis','Condor', RTHandler)
allHandlers.add('TRExPlusOAAnalysis','LCG', LCGRTHandler)
allHandlers.add('TRExPlusOAAnalysis','gLite', gLiteRTHandler)
allHandlers.add('TRExPlusOAAnalysis','TestSubmitter', RTHandler)
allHandlers.add('TRExPlusOAAnalysis','Interactive', RTHandler)
allHandlers.add('TRExPlusOAAnalysis','Batch', RTHandler)
allHandlers.add('TRExPlusOAAnalysis','Cronus', RTHandler)
allHandlers.add('TRExPlusOAAnalysis','Remote', LCGRTHandler)
allHandlers.add('TRExPlusOAAnalysis','CREAM', LCGRTHandler)

